﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DDCF
{
  /// <summary>対数DDモデル（指数平滑モデル）</summary>
  public class LogarithmicDiscreteTimeDiffustionModel
  {

    #region インスタンス変数

    /// <summary>乱数</summary>
    private Random rnd;

    /// <summary>正規乱数のストックがあるか</summary>
    private bool hasStock = false;

    /// <summary>正規乱数のストック</summary>
    private double nRandom;

    /// <summary>現在の値</summary>
    private double currentValue = 1;

    /// <summary>前時点の値</summary>
    private double lastValue;

    #endregion

    #region プロパティ

    /// <summary>トレンドを取得する</summary>
    public double Trend
    {
      private set;
      get;
    }

    /// <summary>分散を取得する</summary>
    public double Deviation
    {
      private set;
      get;
    }

    /// <summary>平滑パラメータを取得する</summary>
    public double Smoothing
    {
      private set;
      get;
    }

    #endregion

    #region コンストラクタ

    /// <summary>コンストラクタ</summary>
    /// <param name="seed">乱数シード</param>
    /// <param name="trend">トレンド（年率）</param>
    /// <param name="deviation">分散（年率）</param>
    /// <param name="smoothing">平滑パラメータ（0～1）。0でトレンド一定</param>
    public LogarithmicDiscreteTimeDiffustionModel
      (int seed, double trend, double deviation, double smoothing, double initialValue)
    {
      rnd = new Random(seed);
      this.Trend = trend;
      this.Deviation = deviation;
      this.Smoothing = smoothing;
      this.currentValue = this.lastValue = initialValue;
    }

    #endregion

    #region その他処理

    /// <summary>次の時点の状態値を計算する</summary>
    /// <returns>次の時点の状態値</returns>
    public double NextValue()
    {

      //トレンドを更新
      if(lastValue != 0)
        Trend = Smoothing * Math.Log(currentValue / lastValue) + (1d - Smoothing) * Trend;

      //正規乱数を作成
      double nRnd;
      if (hasStock)
      {
        nRnd = this.nRandom;
        hasStock = false;
      }
      else
      {
        getNormalRandomNumber(rnd, out nRnd, out nRandom);
        hasStock = true;
      }

      //現在値を保存して更新
      lastValue = currentValue;
      currentValue *= Math.Exp(Trend * 1d/12 + Deviation * Math.Sqrt(1d/12) * nRnd);
      return currentValue;
    }

    /// <summary>BoxMuller法で平均0分散1の正規乱数を計算する</summary>
    /// <param name="rnd">乱数源</param>
    /// <param name="nrnd1">正規乱数1</param>
    /// <param name="nrnd2">正規乱数2</param>
    private static void getNormalRandomNumber
      (Random rnd, out double nrnd1, out double nrnd2)
    {
      double v = rnd.NextDouble() * 2 - 1;
      double u = rnd.NextDouble() * 2 - 1;
      double r2 = v * v + u * u;
      while (1 < r2)
      {
        v = rnd.NextDouble() * 2 - 1;
        u = rnd.NextDouble() * 2 - 1;
        r2 = v * v + u * u;
      }
      double w = Math.Sqrt(-2.0 * Math.Log(r2) / r2);
      nrnd1 = w * v;
      nrnd2 = w * u;
    }

    #endregion

  }
}
