﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.IO;

using Popolo.ThermalLoad;
using Popolo.Weather;
using Popolo.Electric;
using Popolo.HVAC;
using Popolo.HVAC.SubSystem;
using Popolo.HVAC.AirConditioner;
using Popolo.HVAC.HeatSource;
using Popolo.ThermophysicalProperty;
using Popolo.HVAC.Circuit;
using Popolo.HVAC.HeatExchanger;
using Popolo.HumanBody;

using Popolo.BuildingOccupant;
using Popolo.Numerics;

namespace DDCF
{
  class Program
  {

    const bool MONTHLY_OUTPUT = false;

    #region クラス変数

    /// <summary>計算年数</summary>
    private static uint YEARS = 1;

    /// <summary>エネルギー計算を行うか</summary>
    private static bool CALC_ENERGY = true;

    /// <summary>便所水使用量を計算するか</summary>
    private static bool CALC_FLUSH = false;

    /// <summary>発電量を計算するか</summary>
    private static bool CALC_PVPNL = false;

    #endregion

    static void Main(string[] args)
    {
      PerformDDCF();
      //performSingleDDCFCase(1, 1);
      //testVacancyRate();
      //test24HourRate();
      //testStandardCase("test.csv");
    }

    #region テナント入退室関連

    /// <summary>空室率をテストする</summary>
    private static void testVacancyRate()
    {
      const int SEED = 1;         //乱数シード
      const int ITER_NUM = 15000; //分布計算用繰り返し数
      const int MONTHS = 20 * 12;       //計算月数

      int[] schMonth = new int[] { 2, 4, 6, 8, 10, 12, 14, 16, 18 };
      int[] devMonth = new int[] { 4 * 4, 5 * 5, 6 * 6 };
      double[] temProv = new double[] { 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50 };
      int caseNum = schMonth.Length * devMonth.Length * temProv.Length;
      double[][][][] vacRates = new double[schMonth.Length][][][];
      for (int i = 0; i < schMonth.Length; i++)
      {
        vacRates[i] = new double[devMonth.Length][][];
        for (int j = 0; j < devMonth.Length; j++)
        {
          vacRates[i][j] = new double[temProv.Length][];
          for (int k = 0; k < temProv.Length; k++)
            vacRates[i][j][k] = new double[ITER_NUM];
        }
      }

      MersenneTwister nRnd = new MersenneTwister(SEED);
      uint[] seeds = new uint[caseNum];
      for (int i = 0; i < seeds.Length; i++) seeds[i] = nRnd.Next();

      int cNum = 0;
      Parallel.For(0, schMonth.Length, schIndex =>
      {
        Parallel.For(0, devMonth.Length, devIndex =>
        {
          Parallel.For(0, temProv.Length, temIndex =>
          {
            Console.WriteLine("Case:" + schIndex + "-" + devIndex + "-" + temIndex);
            TenantProcess tProc = new TenantProcess(seeds[cNum]);
            lock (schMonth) { tProc.SearchingMonth_Average = schMonth[schIndex]; }
            lock (devMonth) { tProc.SearchingMonth_Deviation = devMonth[devIndex]; }
            lock (temProv) { tProc.TermLeavingProbability = temProv[temIndex]; }

            for (int i = 0; i < ITER_NUM; i++)
            {
              int[] prc = tProc.MakeProcess(MONTHS, 0);
              double sum = 0;
              for (int j = 0; j < prc.Length; j++)
                if (prc[j] == 0) sum++;
              lock (vacRates) vacRates[schIndex][devIndex][temIndex][i] = sum / MONTHS;
            }
          });
        });
      });

      //書き出し
      using (StreamWriter sWriter = new StreamWriter("VacaancyTest.csv"))
      {
        for (int i = 0; i < devMonth.Length; i++)
        {
          for (int j = 0; j < schMonth.Length; j++)
          {
            for (int k = 0; k < temProv.Length; k++)
            {
              sWriter.Write("Case:," + schMonth[j] + "," + devMonth[i] + "," + temProv[k]);
              for (int m = 0; m < ITER_NUM; m++) sWriter.Write("," + vacRates[j][i][k][m]);
              sWriter.WriteLine();
            }
          }
        }
      }
    }

    /// <summary>24時間の在室率をテストする</summary>
    private static void test24HourRate()
    {
      const int ITER_NUM = 5000; //分布計算用繰り返し数
      const int YEARS = 20; //計算年数
      const int T_NUM = 12; //テナント数
      const double F_AREA = 600;  //テナント貸室面積[m2]
      const uint RND_SEED = 1;

      //既存ファイル削除
      if (File.Exists("test24Hour.csv")) File.Delete("test24Hour.csv");
      if (File.Exists("test24Hour2.csv")) File.Delete("test24Hour2.csv"); //debug

      MersenneTwister nRnd = new MersenneTwister(RND_SEED);
      uint[] seeds = new uint[ITER_NUM];
      for (int i = 0; i < seeds.Length; i++) seeds[i] = nRnd.Next();

      //分布計算のための繰り返し
      ParallelOptions options = new ParallelOptions();
      options.MaxDegreeOfParallelism = 4;  //スレッド数
      Parallel.For(0, ITER_NUM,options, cNum =>
      {
        Console.WriteLine("Case:" + cNum);

        //乱数生成器
        MersenneTwister nRnd2;
        lock (seeds) { nRnd2 = new MersenneTwister(seeds[cNum]); }

        //20年間の入退室過程を作成
        OfficeTenant[] tenants = new OfficeTenant[T_NUM];
        int[][] procs = new int[T_NUM][];
        TenantProcess tProc = new TenantProcess(nRnd2.Next());
        //空室率8%相当
        tProc.SearchingMonth_Average = 8;
        tProc.SearchingMonth_Deviation = 6 * 6;
        tProc.TermLeavingProbability = 0.40;
        /*//空室率4%相当
        tProc.SearchingMonth_Average = 6;
        tProc.SearchingMonth_Deviation = 4 * 4;
        tProc.TermLeavingProbability = 0.40;*/
        for (int i = 0; i < T_NUM; i++)
          procs[i] = tProc.MakeProcess(12 * YEARS, 0);
        int[] tNumber = new int[T_NUM];
        for (int i = 0; i < T_NUM; i++) tNumber[i] = -1;

        //20年間の計算実行
        int ttlDays = 0;
        int lstDay = 0;
        int lstMonth = 0;
        int ttlMonth = -1;
        int[] pSum = new int[24];
        DateTime dTime = new DateTime(2000, 1, 1, 0, 0, 0);
        while (true)
        {
          //月が変わった場合
          if (dTime.Month != lstMonth)
          {
            lstMonth = dTime.Month;

            //終了判定
            if (dTime.Year == (2000 + YEARS))
            {
              while (true)
              {
                StreamWriter sWriter = null;
                try
                {
                  sWriter = new StreamWriter("test24Hour.csv", true);
                  sWriter.Write(cNum);
                  for (int i = 0; i < pSum.Length; i++) sWriter.Write("," + (pSum[i] / (double)(ttlDays * F_AREA * T_NUM)));
                  sWriter.WriteLine();
                  break;
                }
                catch { }
                finally
                {
                  if (sWriter != null) sWriter.Close();
                }
              }
              break;
            }
            
            //テナント入退室確認
            ttlMonth++;
            for (int i = 0; i < tNumber.Length; i++)
            {
              //テナント入替時は新規テナントモデルを作成
              if (tNumber[i] != procs[i][ttlMonth])
              {
                tNumber[i] = procs[i][ttlMonth];

                if (procs[i][ttlMonth] == 0) tenants[i] = null;
                else
                {
                  OfficeTenant.CategoryOfIndustry cat = (OfficeTenant.CategoryOfIndustry)Math.Floor(nRnd2.NextDouble() * 8);
                  tenants[i] = new OfficeTenant
                    (cat, F_AREA, OfficeWorker.DaysOfWeek.Saturday & OfficeWorker.DaysOfWeek.Sunday, nRnd2);
                }
              }
            }
          }

          //日にちが変わった場合
          if (dTime.Day != lstDay)
          {
            lstDay = dTime.Day;

            //テナント執務者スケジュールを更新
            for (int i = 0; i < tenants.Length; i++)
              if (tenants[i] != null) tenants[i].UpdateDailySchedule(dTime);
            if (dTime.DayOfWeek != DayOfWeek.Saturday && dTime.DayOfWeek != DayOfWeek.Sunday) ttlDays++;
          }
          
          //執務者人数を集計
          if (dTime.DayOfWeek != DayOfWeek.Saturday && dTime.DayOfWeek != DayOfWeek.Sunday)
          {
            for (int i = 0; i < tenants.Length; i++)
            {
              if (tenants[i] != null)
              {
                tenants[i].UpdateStatus(dTime);
                pSum[dTime.Hour] += (int)tenants[i].StayWorkerNumber;
              }
            }
          }
          
          dTime = dTime.AddHours(1);
        }
      });
    }

    #endregion

    #region SHASE基準ケース検証計算

    /// <summary>SHASE基準ケースのテスト</summary>
    private static void testStandardCase(string outputFile)
    {
      //建物モデル作成
      BuildingThermalModel bModel;
      AHUSystem ahuSystem;
      SHASE_SimulationTest.MakeAHUSystem(out bModel, out ahuSystem, false, true); //ここでVAV切り替え
      Sun sun = new Sun(Sun.City.Tokyo);

      //熱源モデル作成
      HeatSourceSystemModel hss;
      AirHeatSourceModularChillersSystem ahSystem;
      DirectFiredAbsorptionChillerSystem arSystem;
      SHASE_SimulationTest.MakeHeatSourceSystem(out hss, out ahSystem, out arSystem);

      //HVACモデル作成
      HVACSystemModel hvacSystem = new HVACSystemModel
        (bModel, new IAirConditioningSystemModel[] { ahuSystem }, hss);
      hvacSystem.ChilledWaterSupplyTemperatureSetpoint = 7;
      hvacSystem.HotWaterSupplyTemperatureSetpoint = 50;
      hvacSystem.SetACFactor(0, 1); //等倍

      //気象データ読み込み
      double[] dbt, hrt, dnr, dhr, ncr;
      SHASE_SimulationTest.LoadWeatherData("3639999.has", out dbt, out hrt, out dnr, out dhr, out ncr);
      using (StreamWriter sWriter = new StreamWriter(outputFile, false, Encoding.GetEncoding("Shift_JIS")))
      {
        //タイトル行書き出し
        //外界条件
        sWriter.Write(",,外気乾球温度,外気絶対湿度,法線面直達日射,水平面全天日射,夜間放射");
        //一次側
        sWriter.Write(", 冷水往温度, 冷水還温度, 温水往温度, 温水還温度");
        sWriter.Write(", AHP電力, AHP冷水ポンプ電力, AHP温水ポンプ電力, AHP冷却, AHP加熱, AHP台数");
        sWriter.Write(", ARガス, AR電力, AR冷水ポンプ電力, AR温水ポンプ電力");
        sWriter.Write(", AR冷却水ポンプ電力, AR冷却塔電力, AR冷却, AR加熱");
        sWriter.Write(", 冷水二次ポンプ電力, 温水二次ポンプ電力");
        sWriter.Write(", 二次側冷水流量, 二次側温水流量, 冷水バイパス流量, 温水バイパス流量");
        //二次側
        sWriter.Write(", AHUファン動力");
        sWriter.WriteLine();

        //12月を助走計算期間とする
        DateTime dt = new DateTime(2006, 12, 1, 0, 0, 0);
        for (int i = 0; i < 31 * 24; i++)
        {
          //気象条件更新
          sun.SetGlobalHorizontalRadiation(dhr[8016 + i], dnr[8016 + i]);
          sun.Update(dt.AddMinutes(30)); //過去一時間のデータのため30分シフト
          hvacSystem.UpdateOutdoorCondition(dt, sun, dbt[8016 + i], hrt[8016 + i], ncr[8016 + i]);

          //熱平衡を更新
          SHASE_SimulationTest.ControlAHUSystem(bModel, ahuSystem);
          SHASE_SimulationTest.ControlHeatSourceSystem(hss);
          hvacSystem.Update();
          dt = dt.AddHours(1);
        }

        //8760時間の計算実行
        dt = new DateTime(2006, 1, 1, 0, 0, 0);
        for (int i = 0; i < 8760; i++)
        {
          //気象条件更新
          sun.SetGlobalHorizontalRadiation(dhr[i], dnr[i]);
          sun.Update(dt.AddMinutes(30)); //過去一時間のデータのため30分シフト
          hvacSystem.UpdateOutdoorCondition(dt, sun, dbt[i], hrt[i], ncr[i]);

          //状態更新
          SHASE_SimulationTest.ControlAHUSystem(bModel, ahuSystem);
          SHASE_SimulationTest.ControlHeatSourceSystem(hss);
          hvacSystem.Update();

          //書き出し
          //外界条件
          sWriter.Write(dt.ToShortDateString() + "," + dt.ToShortTimeString());
          sWriter.Write("," + dbt[i] + "," + hrt[i] + "," + dnr[i] + "," + dhr[i] + "," + ncr[i]);
          //一次側
          sWriter.Write(
            "," + hvacSystem.ChilledWaterSupplyTemperature +
            "," + hvacSystem.ChilledWaterReturnTemperature +
            "," + hvacSystem.HotWaterSupplyTemperature +
            "," + hvacSystem.HotWaterReturnTemperature);
          ImmutableAirHeatSourceModularChillers ahp = ahSystem.AirHeatSourceModularChillers;
          sWriter.Write(
            "," + (ahp.ElectricConsumption * ahp.OperatingNumber) +
            "," + (ahSystem.ChilledWaterPump.GetElectricConsumption() +
            "," + ahSystem.HotWaterPump.GetElectricConsumption()) +
            "," + ahp.CoolingLoad + "," + ahp.HeatingLoad + "," + ahSystem.OperatingChillerNumber);
          ImmutableDirectFiredAbsorptionChiller ar = arSystem.DirectFiredAbsorptionChiller;
          sWriter.Write(
            "," + ar.FuelConsumption + "," + ar.ElectricConsumption +
            "," + (arSystem.ChilledWaterPump.GetElectricConsumption() +
            "," + arSystem.HotWaterPump.GetElectricConsumption()) +
            "," + arSystem.CoolingWaterPump.GetElectricConsumption() +
            "," + arSystem.CoolingTower.ElectricConsumption +
            "," + ar.CoolingLoad + "," + ar.HeatingLoad);
          ImmutablePumpSystem ps2c = hss.ChilledWaterPumpSystem;
          ImmutablePumpSystem ps2h = hss.HotWaterPumpSystem;
          sWriter.Write(
            "," + ps2c.GetElectricConsumption() + "," + ps2h.GetElectricConsumption() +
            "," + (ps2c.TotalFlowRate * 1000) + "," + (ps2h.TotalFlowRate * 1000) +
            "," + hss.ChilledWaterBypassFlowRate + "," + hss.HotWaterBypassFlowRate);
          //二次側
          double ahuEsum = 0;
          for (int j = 0; j < ahuSystem.AHUs.Length; j++)
            ahuEsum += ahuSystem.AHUs[j].SupplyAirFan.GetElectricConsumption() + ahuSystem.AHUs[j].ReturnAirFan.GetElectricConsumption();
          sWriter.Write("," + ahuEsum);
          sWriter.WriteLine();

          if (dt.Hour == 0) Console.WriteLine(dt.ToShortDateString());
          dt = dt.AddHours(1);
        }
      }

    }

    #endregion

    #region DDCF計算

    private static void PerformDDCF()
    {
      //計算条件読み込み
      int parNum, iterNum, iterStt, iterEnd;
      uint rndSeed;
      using (StreamReader sReader = new StreamReader("setting.ini"))
      {
        parNum = int.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        rndSeed = uint.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        YEARS = uint.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        iterNum = int.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        iterStt = int.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        iterEnd = int.Parse(sReader.ReadLine().Split(';')[0].Split('=')[1]);
        CALC_ENERGY = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
        CALC_FLUSH = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
        CALC_PVPNL = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
        SHASE_SimulationTest.IS_HIGHEFF_HSOURCE = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
        SHASE_SimulationTest.USE_REGENERATOR = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
        SHASE_SimulationTest.USE_CO2CNTRL = (sReader.ReadLine().Split(';')[0].Split('=')[1] == "1");
      }

      //熱負荷計算自体の並列化は停止
      BuildingThermalModel.EnableParallelComputing = false;

      //既存ファイル削除
      if (File.Exists("DDCF_Tenant.csv")) File.Delete("DDCF_Tenant.csv");
      if (File.Exists("DDCF_Occupant.csv")) File.Delete("DDCF_Occupant.csv");
      if (File.Exists("DDCF_PV.csv")) File.Delete("DDCF_PV.csv");
      if (File.Exists("DDCF_Elec.csv")) File.Delete("DDCF_Elec.csv");
      if (File.Exists("DDCF_Gas.csv")) File.Delete("DDCF_Gas.csv");
      if (File.Exists("DDCF_Water.csv")) File.Delete("DDCF_Water.csv");
      if (File.Exists("DDCF_Sewage.csv")) File.Delete("DDCF_Sewage.csv");

      //乱数シード作成
      MersenneTwister nRnd = new MersenneTwister(rndSeed);
      uint[] seeds = new uint[iterNum];
      for (int i = 0; i < seeds.Length; i++) seeds[i] = nRnd.Next();

      //分布計算のための繰り返し
      ParallelOptions options = new ParallelOptions();
      options.MaxDegreeOfParallelism = parNum;  //スレッド数
      int indx = 0;
      Parallel.For(iterStt, iterEnd, options, cNum =>
      {
        uint sd;
        int caseNumber;
        lock (seeds)
        {
          caseNumber = indx + iterStt;
          sd = seeds[caseNumber];
          indx++;
        }
        try
        {
          performSingleDDCFCase(sd, caseNumber);
        }
        catch
        {
          Console.WriteLine(caseNumber + ":Error");
        }
      });
    }

    /// <summary>DDCFのための1ケース計算</summary>
    private static void performSingleDDCFCase(uint seed, int caseNumber)
    {
      if (File.Exists("debug.csv")) File.Delete("debug.csv");

      //乱数初期化
      MersenneTwister uRnd = new MersenneTwister(seed);
      MersenneTwister wRnd = new MersenneTwister(uRnd.Next());  //気象データ
      TenantProcess tProc = new TenantProcess(uRnd.Next()); //テナントプロセス

      //建物モデル作成
      BuildingThermalModel bModel;
      AHUSystem ahuSystem;
      SHASE_SimulationTest.MakeAHUSystem(out bModel, out ahuSystem, false, false); //ここでVAV切り替え
      Sun sun = new Sun(Sun.City.Tokyo);

      //熱源モデル作成
      HeatSourceSystemModel hss;
      AirHeatSourceModularChillersSystem ahSystem;
      DirectFiredAbsorptionChillerSystem arSystem;
      SHASE_SimulationTest.MakeHeatSourceSystem(out hss, out ahSystem, out arSystem);

      //HVACモデル作成
      HVACSystemModel hvacSystem = new HVACSystemModel
        (bModel, new IAirConditioningSystemModel[] { ahuSystem }, hss);
      hvacSystem.ChilledWaterSupplyTemperatureSetpoint = 7;
      hvacSystem.HotWaterSupplyTemperatureSetpoint = 50;
      hvacSystem.SetACFactor(0, 1); //等倍

      //太陽光発電パネル作成
      PhotovoltaicPanel pvPanel = new PhotovoltaicPanel
        (1000, PhotovoltaicPanel.MountType.MountMode, PhotovoltaicPanel.MaterialType.Amorphous, 30);

      //気象データ作成
      double[] dbt, hrt, dnr, dhr, ncr;
      SHASE_SimulationTest.MakeWeatherData(wRnd.Next(), out dbt, out hrt, out dnr, out dhr, out ncr);

      //内部発熱を追加
      TenantLoad[] tLoads = new TenantLoad[12];
      for (int i = 0; i < tLoads.Length; i++) tLoads[i] = new TenantLoad(null);
      for (int fl = 0; fl < 6; fl++)
      {
        for (int zn = 0; zn < bModel.MultiRoom[fl * 3].ZoneNumber; zn++)
          bModel.AddHeatGain(fl * 3, zn, tLoads[2 * fl]);
        for (int zn = 0; zn < bModel.MultiRoom[fl * 3 + 1].ZoneNumber; zn++)
          bModel.AddHeatGain(fl * 3 + 1, zn, tLoads[2 * fl + 1]);
      }

      //各テナントの入退室過程を作成
      int[][] procs = new int[12][];
      //空室率8%相当
      tProc.SearchingMonth_Average = 8;
      tProc.SearchingMonth_Deviation = 6 * 6;
      tProc.TermLeavingProbability = 0.40;
      for (int i = 0; i < procs.Length; i++)
      {
        int mnth;
        if (uRnd.NextDouble() < 0.08) mnth = -1;
        else mnth = (int)Math.Floor(24 * uRnd.NextDouble());
        procs[i] = tProc.MakeProcess(24 * YEARS, mnth);
      }
      int[] tNumber = new int[procs.Length];
      for (int i = 0; i < tNumber.Length; i++) tNumber[i] = -1;

      //助走計算期間を実行（12月最終週）
      const int PREV_RUN = 7;
      int strt = (365 - PREV_RUN) * 24;
      DateTime dTime = new DateTime(2001, 1, 1, 0, 0, 0);
      if (CALC_ENERGY)
      {
        for (int i = 0; i < PREV_RUN * 24; i++)
        {
          //気象条件更新
          sun.SetGlobalHorizontalRadiation(dhr[strt + i], dnr[strt + i]);
          sun.Update(dTime.AddMinutes(30)); //過去一時間のデータのため30分シフト
          hvacSystem.UpdateOutdoorCondition(dTime, sun, dbt[strt + i], hrt[strt + i], ncr[strt + i]);

          //熱平衡を更新
          SHASE_SimulationTest.ControlAHUSystem(bModel, ahuSystem);
          SHASE_SimulationTest.ControlHeatSourceSystem(hss);
          hvacSystem.Update();
          dTime = dTime.AddHours(1);
        }
      }

      //YEARS年間の計算実行
      int ttlHour = 0;
      int ttlYear = -1;
      int lstMonth = 0;
      int ttlMonth = -1;
      double[] tntNum = new double[YEARS * 12];
      double[] ocpSum = new double[YEARS * 12];
      double[] pvSum = new double[YEARS * 12];
      double[] elcSum = new double[YEARS * 12];
      double[] gasSum = new double[YEARS * 12];
      double[] watSum = new double[YEARS * 12];
      double[] swgSum = new double[YEARS * 12];
      while (true)
      {
        //年が変わった場合には気象データ更新+日時初期化////////////////////////////////////////////////////
        if (dTime.Year != 1999)
        {
          ttlYear++;
          if(!MONTHLY_OUTPUT) Console.WriteLine("Case " + caseNumber + ":" + ttlYear);

          //終了判定
          if (YEARS == ttlYear)
          {
            outputResult("DDCF_Tenant.csv", caseNumber, tntNum);
            outputResult("DDCF_Occupant.csv", caseNumber, ocpSum);
            outputResult("DDCF_PV.csv", caseNumber, pvSum);
            outputResult("DDCF_Elec.csv", caseNumber, elcSum);
            outputResult("DDCF_Gas.csv", caseNumber, gasSum);
            outputResult("DDCF_Water.csv", caseNumber, watSum);
            outputResult("DDCF_Sewage.csv", caseNumber, swgSum);
            return;
          }

          //気象データ更新
          SHASE_SimulationTest.MakeWeatherData(wRnd.Next(), out dbt, out hrt, out dnr, out dhr, out ncr);

          //日時初期化//うるう年以外の8760時間の年
          dTime = new DateTime(1999, 1, 1, 0, 0, 0);
          ttlHour = 0;
        }

        //月が変わった場合にはテナント情報更新/////////////////////////////////////////////////////////////
        if (dTime.Month != lstMonth)
        {
          lstMonth = dTime.Month;

          ttlMonth++;
          if(MONTHLY_OUTPUT) Console.WriteLine("Case " + caseNumber + ":" + ttlMonth);

          //テナント入退室確認
          ocpSum[ttlMonth] = 0;
          tntNum[ttlMonth] = 0;
          for (int i = 0; i < tNumber.Length; i++)
          {
            //テナント入替時は新規テナントモデルを作成
            if (tNumber[i] != procs[i][ttlMonth])
            {
              tNumber[i] = procs[i][ttlMonth];

              if (procs[i][ttlMonth] == 0) tLoads[i].Tenant = null;
              else
              {
                OfficeTenant.CategoryOfIndustry cat = (OfficeTenant.CategoryOfIndustry)Math.Floor(uRnd.NextDouble() * 8);
                double fArea;
                if(i % 2 == 0) fArea = 292.5 + 57.5 + 187.5 + 57.5; //北側テナント
                else fArea = 279.0 + 57.5 + 167.5;                  //南側テナント
                OfficeTenant tnt = new OfficeTenant
                  (cat, fArea, OfficeWorker.DaysOfWeek.Saturday & OfficeWorker.DaysOfWeek.Sunday, new MersenneTwister(uRnd.Next()));
                tLoads[i].Tenant = tnt;
              }
            }
            if (tLoads[i].Tenant != null)
            {
              tntNum[ttlMonth]++;
              ocpSum[ttlMonth] += tLoads[i].Tenant.OfficeWorkerNumber;
            }
          }
        }

        //時刻毎にエネルギー計算モデルを更新///////////////////////////////////////////////////////////////
        //気象条件更新
        sun.SetGlobalHorizontalRadiation(dhr[ttlHour], dnr[ttlHour]);
        sun.Update(dTime.AddMinutes(30)); //過去一時間のデータのため30分シフト

        //エネルギー計算
        if (CALC_ENERGY)
        {
          hvacSystem.UpdateOutdoorCondition(dTime, sun, dbt[ttlHour], hrt[ttlHour], ncr[ttlHour]);
          SHASE_SimulationTest.ControlAHUSystem_NoMidSeason(bModel, ahuSystem);
          //SHASE_SimulationTest.ControlAHUSystem(bModel, ahuSystem);
          SHASE_SimulationTest.ApplyCO2Control(ahuSystem, tLoads);
          //テナントがいない場合には空調停止
          for (int i = 0; i < tLoads.Length; i++)
            if (tLoads[i].Tenant == null) ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.ShutOff;
          SHASE_SimulationTest.ControlHeatSourceSystem(hss);
          hvacSystem.Update();
          //エネルギー・水資源積算
          double bf1, bf2, bf3, bf4;
          integrateEnergy(hss, ahuSystem, ahSystem, arSystem, out bf1, out bf2, out bf3, out bf4);
          elcSum[ttlMonth] += bf1;
          gasSum[ttlMonth] += bf2;
          watSum[ttlMonth] += bf3;
          swgSum[ttlMonth] += bf4;
        }

        //便所洗浄水の計算
        if (CALC_FLUSH)
        {
          //1日に1回計算する
          if (dTime.Hour == 0)
          {
            double wSum = 0;
            for (int i = 0; i < tLoads.Length; i++)
            {
              tLoads[i].UpdateDailySchedule(dTime);
              wSum += tLoads[i].CalculateFlushWater();
            }
            watSum[ttlMonth] += wSum;
            swgSum[ttlMonth] += wSum;
          }
        }

        //太陽光発電の計算
        if (CALC_PVPNL)
          pvSum[ttlMonth] += 0.001 * pvPanel.GetPower(dbt[ttlHour], 3, sun);

        //debug書き出し*******************************************
        /*using (StreamWriter sWriter = new StreamWriter("debug.csv", true))
        {
          //外界条件
          sWriter.Write(dTime.ToShortDateString() + "," + dTime.ToShortTimeString());
          sWriter.Write("," + dbt[ttlHour] + "," + hrt[ttlHour] + "," + dnr[ttlHour] + "," + dhr[ttlHour] + "," + ncr[ttlHour]);
          //一次側
          sWriter.Write(
            "," + hvacSystem.ChilledWaterSupplyTemperature +
            "," + hvacSystem.ChilledWaterReturnTemperature +
            "," + hvacSystem.HotWaterSupplyTemperature +
            "," + hvacSystem.HotWaterReturnTemperature);
          ImmutableAirHeatSourceModularChillers ahp = ahSystem.AirHeatSourceModularChillers;
          sWriter.Write(
            "," + (ahp.ElectricConsumption * ahp.OperatingNumber) +
            "," + (ahSystem.ChilledWaterPump.GetElectricConsumption() +
            "," + ahSystem.HotWaterPump.GetElectricConsumption()) +
            "," + ahp.CoolingLoad + "," + ahp.HeatingLoad + "," + ahSystem.OperatingChillerNumber);
          ImmutableDirectFiredAbsorptionChiller ar = arSystem.DirectFiredAbsorptionChiller;
          sWriter.Write(
            "," + ar.FuelConsumption + "," + ar.ElectricConsumption +
            "," + (arSystem.ChilledWaterPump.GetElectricConsumption() +
            "," + arSystem.HotWaterPump.GetElectricConsumption()) +
            "," + arSystem.CoolingWaterPump.GetElectricConsumption() +
            "," + arSystem.CoolingTower.ElectricConsumption +
            "," + ar.CoolingLoad + "," + ar.HeatingLoad);
          ImmutablePumpSystem ps2c = hss.ChilledWaterPumpSystem;
          ImmutablePumpSystem ps2h = hss.HotWaterPumpSystem;
          sWriter.Write(
            "," + ps2c.GetElectricConsumption() + "," + ps2h.GetElectricConsumption() +
            "," + (ps2c.TotalFlowRate * 1000) + "," + (ps2h.TotalFlowRate * 1000) +
            "," + hss.ChilledWaterBypassFlowRate + "," + hss.HotWaterBypassFlowRate);
          //二次側
          double ahuEsum = 0;
          for (int j = 0; j < ahuSystem.AHUs.Length; j++)
            ahuEsum += ahuSystem.AHUs[j].SupplyAirFan.GetElectricConsumption() + ahuSystem.AHUs[j].ReturnAirFan.GetElectricConsumption();
          sWriter.Write("," + ahuEsum);
          sWriter.Write("," + ahuSystem.AHUs[0].Regenerator.SupplyAirOutletDrybulbTemperature
            + "," + ahuSystem.AHUs[0].Regenerator.SupplyAirOutletHumidityRatio
            + "," + ahuSystem.AHUs[0].Regenerator.GetHeatRecovery());
          sWriter.WriteLine();
        }
        //*****************************************************************************/


        //1時間進める//////////////////////////////////////////////////////////////////////////////////////
        //Console.WriteLine(dTime);
        dTime = dTime.AddHours(1);
        ttlHour++;
      }
    }

    /// <summary>エネルギー消費量を積算する</summary>
    /// <param name="hss">熱源システム</param>
    /// <param name="ahuSystem">空調システム</param>
    /// <param name="ahSystem">空気熱源HPシステム</param>
    /// <param name="arSystem">直焚吸収冷温水器システム</param>
    /// <param name="elcSum">電力消費量[kWh]</param>
    /// <param name="gasSum">ガス消費量[Nm3]</param>
    /// <param name="watSum">給水量[m3]</param>
    /// <param name="swgSum">排水量[m3]</param>
    private static void integrateEnergy
      (HeatSourceSystemModel hss, AHUSystem ahuSystem, AirHeatSourceModularChillersSystem ahSystem, DirectFiredAbsorptionChillerSystem arSystem,
      out double elcSum, out double gasSum, out double watSum, out double swgSum)
    {      
      //空気熱源関連
      ImmutableAirHeatSourceModularChillers ahp = ahSystem.AirHeatSourceModularChillers;
      elcSum = (ahp.ElectricConsumption * ahp.OperatingNumber * ahSystem.OperatingChillerNumber)
        + ahSystem.ChilledWaterPump.GetElectricConsumption()
        + ahSystem.HotWaterPump.GetElectricConsumption();

      //直焚関連
      ImmutableDirectFiredAbsorptionChiller ar = arSystem.DirectFiredAbsorptionChiller;
      elcSum += ar.ElectricConsumption
        + arSystem.ChilledWaterPump.GetElectricConsumption()
        + arSystem.HotWaterPump.GetElectricConsumption()
        + arSystem.CoolingWaterPump.GetElectricConsumption()
        + arSystem.CoolingTower.ElectricConsumption;
      gasSum = ar.FuelConsumption * 3600; // m3/h
      watSum = arSystem.CoolingTower.WaterConsumption * 3.6; // m3/h
      swgSum = arSystem.CoolingTower.BlowDownWater * 3.6;    // m3/h

      //二次ポンプ関連
      ImmutablePumpSystem ps2c = hss.ChilledWaterPumpSystem;
      ImmutablePumpSystem ps2h = hss.HotWaterPumpSystem;
      elcSum += ps2c.GetElectricConsumption() + ps2h.GetElectricConsumption();

      //空気搬送関連
      for (int j = 0; j < ahuSystem.AHUs.Length; j++)
      {
        elcSum += ahuSystem.AHUs[j].SupplyAirFan.GetElectricConsumption() + ahuSystem.AHUs[j].ReturnAirFan.GetElectricConsumption();
        watSum += ahuSystem.AHUs[j].WaterConsumption * 3.6; // m3/h
      }
    }

    /// <summary>計算結果を書き出す（1ケース）</summary>
    /// <param name="fileName">書き出しファイル名称</param>
    /// <param name="caseNumber">ケース番号</param>
    /// <param name="data">書き出しデータ</param>
    private static void outputResult(string fileName, int caseNumber, double[] data)
    {
      while (true)
      {
        StreamWriter sWriter = null;
        try
        {
          sWriter = new StreamWriter(fileName, true);
          sWriter.Write(caseNumber);
          for (int i = 0; i < data.Length; i++) sWriter.Write("," + data[i]);
          sWriter.WriteLine();
          break;
        }
        catch { }
        finally
        {
          if (sWriter != null) sWriter.Close();
        }
      }
    }

    //debug書き出し*******************************************
    /*using (StreamWriter sWriter = new StreamWriter("debug.csv", true))
    {
      //外界条件
      sWriter.Write(dTime.ToShortDateString() + "," + dTime.ToShortTimeString());
      sWriter.Write("," + dbt[ttlHour] + "," + hrt[ttlHour] + "," + dnr[ttlHour] + "," + dhr[ttlHour] + "," + ncr[ttlHour]);
      //一次側
      sWriter.Write(
        "," + hvacSystem.ChilledWaterSupplyTemperature +
        "," + hvacSystem.ChilledWaterReturnTemperature +
        "," + hvacSystem.HotWaterSupplyTemperature +
        "," + hvacSystem.HotWaterReturnTemperature);
      ImmutableAirHeatSourceModularChillers ahp = ahSystem.AirHeatSourceModularChillers;
      sWriter.Write(
        "," + (ahp.ElectricConsumption * ahp.OperatingNumber) +
        "," + (ahSystem.ChilledWaterPump.GetElectricConsumption() +
        "," + ahSystem.HotWaterPump.GetElectricConsumption()) +
        "," + ahp.CoolingLoad + "," + ahp.HeatingLoad + "," + ahSystem.OperatingChillerNumber);
      ImmutableDirectFiredAbsorptionChiller ar = arSystem.DirectFiredAbsorptionChiller;
      sWriter.Write(
        "," + ar.FuelConsumption + "," + ar.ElectricConsumption +
        "," + (arSystem.ChilledWaterPump.GetElectricConsumption() +
        "," + arSystem.HotWaterPump.GetElectricConsumption()) +
        "," + arSystem.CoolingWaterPump.GetElectricConsumption() +
        "," + arSystem.CoolingTower.ElectricConsumption +
        "," + ar.CoolingLoad + "," + ar.HeatingLoad);
      ImmutablePumpSystem ps2c = hss.ChilledWaterPumpSystem;
      ImmutablePumpSystem ps2h = hss.HotWaterPumpSystem;
      sWriter.Write(
        "," + ps2c.GetElectricConsumption() + "," + ps2h.GetElectricConsumption() +
        "," + (ps2c.TotalFlowRate * 1000) + "," + (ps2h.TotalFlowRate * 1000) +
        "," + hss.ChilledWaterBypassFlowRate + "," + hss.HotWaterBypassFlowRate);
      //二次側
      double ahuEsum = 0;
      for (int j = 0; j < ahuSystem.AHUs.Length; j++)
        ahuEsum += ahuSystem.AHUs[j].SupplyAirFan.GetElectricConsumption() + ahuSystem.AHUs[j].ReturnAirFan.GetElectricConsumption();
      sWriter.Write("," + ahuEsum);
      sWriter.WriteLine();
    }
    //*****************************************************************************/

    #endregion

  }
}
