﻿using System;
using System.IO;

using Popolo.ThermalLoad;
using Popolo.Weather;
using Popolo.HVAC.AirConditioner;
using Popolo.HVAC.SubSystem;
using Popolo.HVAC.HeatExchanger;
using Popolo.HVAC.Circuit;
using Popolo.HVAC.HeatSource;
using Popolo.ThermophysicalProperty;

namespace DDCF
{
  /// <summary>SHASEシミュレーション評価法テストクラス</summary>
  public static class SHASE_SimulationTest
  {

    #region 定数宣言

    /// <summary>階数</summary>
    public static uint FLR_NUM = 6;

    /// <summary>高効率熱源導入の真偽</summary>
    public static bool IS_HIGHEFF_HSOURCE = false;

    /// <summary>高断熱仕様か否か</summary>
    public static bool IS_HIGH_INSULATION = false;

    /// <summary>CO2制御実施の真偽</summary>
    public static bool USE_CO2CNTRL = false;

    /// <summary>外気冷房の真偽</summary>
    public static bool USE_OA_COOLING = false;

    /// <summary>全熱交換器導入の真偽</summary>
    public static bool USE_REGENERATOR = false;

    /// <summary>VWV制御導入の真偽</summary>
    public static bool USE_VWV_SYSTEM = false;

    #endregion

    #region 熱負荷テスト関連

    /// <summary>建物モデルを作成する</summary>
    /// <param name="addSimpleHeatGain">固定熱負荷を追加するか否か</param>
    /// <returns>建物モデル</returns>
    public static BuildingThermalModel MakeBuildingThermalModel(bool addSimpleHeatGain)
    {
      //傾斜面の作成（四方位）//////////////
      Incline incN = new Incline(Incline.Orientation.N, 0.5 * Math.PI);
      Incline incE = new Incline(Incline.Orientation.E, 0.5 * Math.PI);
      Incline incW = new Incline(Incline.Orientation.W, 0.5 * Math.PI);
      Incline incS = new Incline(Incline.Orientation.S, 0.5 * Math.PI);

      //壁構成を作成////////////////////////
      WallLayer[] exWL = new WallLayer[6];  //外壁一般部分
      exWL[0] = new WallLayer("タイル", 1.3, 2000, 0.010);
      exWL[1] = new WallLayer("セメント・モルタル", 1.5, 1600, 0.025);
      exWL[2] = new WallLayer("コンクリート", 1.6, 2000, 0.150);
      if(IS_HIGH_INSULATION) exWL[3] = new WallLayer("押出ポリスチレンフォーム3種", 0.028, 33, 0.025);
      else exWL[3] = new WallLayer("押出ポリスチレンフォーム1種", 0.040, 33, 0.025);
      exWL[4] = new AirGapLayer("非密閉中空層", false, 0.05);
      exWL[5] = new WallLayer("石膏ボード", 0.22, 830, 0.008);

      WallLayer[] exbmWL = new WallLayer[4];  //外壁梁部分
      exbmWL[0] = new WallLayer("タイル", 1.3, 2000, 0.010);
      exbmWL[1] = new WallLayer("セメント・モルタル", 1.5, 1600, 0.025);
      exbmWL[2] = new WallLayer("コンクリート", 1.6, 2000, 0.750);
      if (IS_HIGH_INSULATION) exbmWL[3] = new WallLayer("押出ポリスチレンフォーム3種", 0.028, 33, 0.025);
      else exbmWL[3] = new WallLayer("押出ポリスチレンフォーム1種", 0.040, 33, 0.025);

      WallLayer[] flWL = new WallLayer[6];  //床・天井
      flWL[0] = new WallLayer("ビニル系床材", 0.190, 2000, 0.003);
      flWL[1] = new AirGapLayer("非密閉中空層", false, 0.05);
      flWL[2] = new WallLayer("コンクリート", 1.6, 2000, 0.150);
      flWL[3] = new AirGapLayer("非密閉中空層", false, 0.05);
      flWL[4] = new WallLayer("石膏ボード", 0.220, 830, 0.009);
      flWL[5] = new WallLayer("ロックウール化粧吸音板", 0.064, 290, 0.015);

      WallLayer[] inWL = new WallLayer[3];  //内壁
      inWL[0] = new WallLayer("石膏ボード", 0.220, 830, 0.012);
      inWL[1] = new AirGapLayer("非密閉中空層", false, 0.05);
      inWL[2] = new WallLayer("石膏ボード", 0.220, 830, 0.012);

      //ゾーンを作成/////////////////////////
      Zone[][] zn0 = new Zone[FLR_NUM][];
      Zone[][] zn1 = new Zone[FLR_NUM][];
      Zone[][] zn2 = new Zone[FLR_NUM][];

      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        zn0[fl] = new Zone[4];
        zn1[fl] = new Zone[3];
        zn2[fl] = new Zone[4];
        zn0[fl][0] = new Zone("NI", 292.5 * 2.7 * 1.2, 292.5);
        zn0[fl][1] = new Zone("NWP", 57.5 * 2.7 * 1.2, 57.5);
        zn0[fl][2] = new Zone("NP", 187.5 * 2.7 * 1.2, 187.5);
        zn0[fl][3] = new Zone("NEP", 57.5 * 2.7 * 1.2, 57.5);
        zn1[fl][0] = new Zone("SI", 279.0 * 2.7 * 1.2, 279.0);
        zn1[fl][1] = new Zone("SWP", 57.5 * 2.7 * 1.2, 57.5);
        zn1[fl][2] = new Zone("SP", 167.5 * 2.7 * 1.2, 167.5);
        zn2[fl][0] = new Zone("EV", 12.0 * 2.6 * 1.2, 12.0);
        zn2[fl][1] = new Zone("IL", 144.0 * 2.4 * 1.2, 144.0);
        zn2[fl][2] = new Zone("MC", 21.8 * 2.4 * 1.2, 21.8);
        zn2[fl][3] = new Zone("WC", 69.2 * 2.4 * 1.2, 69.2);

        //内部発熱を設定
        if (addSimpleHeatGain)
        {
          zn0[fl][0].AddHeatGain(new MyHeatGain(0.1 * 292.5, 12 * 292.5, 12 * 292.5, true));
          zn0[fl][1].AddHeatGain(new MyHeatGain(0.1 * 57.5, 12 * 57.5, 12 * 57.5, true));
          zn0[fl][2].AddHeatGain(new MyHeatGain(0.1 * 187.5, 12 * 187.5, 12 * 187.5, true));
          zn0[fl][3].AddHeatGain(new MyHeatGain(0.1 * 57.5, 12 * 57.5, 12 * 57.5, true));
          zn1[fl][0].AddHeatGain(new MyHeatGain(0.1 * 279.0, 12 * 279.0, 12 * 279.0, true));
          zn1[fl][1].AddHeatGain(new MyHeatGain(0.1 * 57.5, 12 * 57.5, 12 * 57.5, true));
          zn1[fl][2].AddHeatGain(new MyHeatGain(0.1 * 167.5, 12 * 167.5, 12 * 167.5, true));
        }
        zn2[fl][0].AddHeatGain(new MyHeatGain(0, 15 * 12.0, 0, false));
        zn2[fl][1].AddHeatGain(new MyHeatGain(0, 15 * 144.0, 0, false));
        zn2[fl][2].AddHeatGain(new MyHeatGain(0, 15 * 21.8, 0, false));
        zn2[fl][3].AddHeatGain(new MyHeatGain(0, 15 * 69.2, 0, false));

        ////隙間風と熱容量設定
        for (int i = 0; i < zn0[fl].Length; i++)
        {
          zn0[fl][i].HeatCapacity = zn0[fl][i].AirMass * 1006 * 10;
          zn0[fl][i].VentilationRate = zn0[fl][i].AirMass / 3600d * 0.1;
          zn0[fl][i].InitializeAirState(22, 0.0105);
        }
        for (int i = 0; i < zn1[fl].Length; i++)
        {
          zn1[fl][i].HeatCapacity = zn1[fl][i].AirMass * 1006 * 10;
          zn1[fl][i].VentilationRate = zn1[fl][i].AirMass / 3600d * 0.1;
          zn1[fl][i].InitializeAirState(22, 0.0105);
        }
        for (int i = 0; i < zn0[fl].Length; i++)
        {
          zn2[fl][i].HeatCapacity = 0;
          zn2[fl][i].VentilationRate = zn2[fl][i].AirMass / 3600d * 0.1;
          zn2[fl][i].InitializeAirState(22, 0.0105);
        }

      }

      //壁体の作成//////////////////////////
      Wall[][] walls = new Wall[FLR_NUM][];
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        walls[fl] = new Wall[49];
        walls[fl][0] = new Wall(32.8, exWL);
        walls[fl][1] = new Wall(12.6, exbmWL);
        walls[fl][2] = new Wall(94.5, exWL);
        walls[fl][3] = new Wall(38.3, exbmWL);
        walls[fl][4] = new Wall(32.8, exWL);
        walls[fl][5] = new Wall(12.6, exbmWL);
        walls[fl][6] = new Wall(32.8, exWL);
        walls[fl][7] = new Wall(12.6, exbmWL);
        walls[fl][8] = new Wall(79.6, exWL);
        walls[fl][9] = new Wall(32.4, exbmWL);
        walls[fl][10] = new Wall(16.8, exWL);
        walls[fl][11] = new Wall(42.6, exWL);
        walls[fl][12] = new Wall(26.0, exWL);
        walls[fl][13] = new Wall(292.5, flWL);
        walls[fl][14] = new Wall(292.5, flWL);
        walls[fl][15] = new Wall(16.2, inWL);
        walls[fl][16] = new Wall(8.1, inWL);
        walls[fl][17] = new Wall(64.8, inWL);
        walls[fl][18] = new Wall(57.5, flWL);
        walls[fl][19] = new Wall(57.5, flWL);
        walls[fl][20] = new Wall(13.5, inWL);
        walls[fl][21] = new Wall(187.5, flWL);
        walls[fl][22] = new Wall(187.5, flWL);
        walls[fl][23] = new Wall(57.5, flWL);
        walls[fl][24] = new Wall(57.5, flWL);
        walls[fl][25] = new Wall(13.5, inWL);
        walls[fl][26] = new Wall(79.7, inWL);
        walls[fl][27] = new Wall(8.0, inWL);
        walls[fl][28] = new Wall(28.4, inWL);
        walls[fl][29] = new Wall(279.0, flWL);
        walls[fl][30] = new Wall(279.0, flWL);
        walls[fl][31] = new Wall(57.5, flWL);
        walls[fl][32] = new Wall(57.5, flWL);
        walls[fl][33] = new Wall(13.5, inWL);
        walls[fl][34] = new Wall(167.5, flWL);
        walls[fl][35] = new Wall(167.5, flWL);
        walls[fl][36] = new Wall(13.5, inWL);
        walls[fl][37] = new Wall(31.2, inWL);
        walls[fl][38] = new Wall(12.0, flWL);
        walls[fl][39] = new Wall(12.0, flWL);
        walls[fl][40] = new Wall(10.8, inWL);
        walls[fl][41] = new Wall(158.4, inWL);
        walls[fl][42] = new Wall(144.0, flWL);
        walls[fl][43] = new Wall(144.0, flWL);
        walls[fl][44] = new Wall(16.9, inWL);
        walls[fl][45] = new Wall(21.8, flWL);
        walls[fl][46] = new Wall(21.8, flWL);
        walls[fl][47] = new Wall(69.2, flWL);
        walls[fl][48] = new Wall(69.2, flWL);

        //壁の初期化
        for (int i = 0; i < 49; i++)
        {
          walls[fl][i].ShortWaveAbsorptanceF = walls[fl][i].ShortWaveAbsorptanceB = 0.8;
          walls[fl][i].LongWaveEmissivityF = walls[fl][i].LongWaveEmissivityB = 0.9;
          walls[fl][i].RadiativeCoefficientF = walls[fl][i].RadiativeCoefficientB = 5;
          if (i <= 12) walls[fl][i].ConvectiveCoefficientF = 18;
          else walls[fl][i].ConvectiveCoefficientF = 4;
          walls[fl][i].ConvectiveCoefficientB = 4;
          walls[fl][i].Initialize(20);
        }
      }

      //窓を作成
      double[] TAU_WIN, RHO_WIN;
      if (IS_HIGH_INSULATION)
      {
        TAU_WIN = new double[] { 0.627, 0.815 }; //ガラスの透過率リスト[-]
        RHO_WIN = new double[] { 0.211, 0.072 }; //ガラスの反射率リスト[-]
      }
      else
      {
        TAU_WIN = new double[] { 0.815 }; //ガラスの透過率リスト[-]
        RHO_WIN = new double[] { 0.072 }; //ガラスの反射率リスト[-]
      }
      Window[][] win = new Window[FLR_NUM * 3][];

      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 3;
        win[shft + 0] = new Window[3];
        win[shft + 1] = new Window[2];
        win[shft + 2] = new Window[0];
        win[shft + 0][0] = new Window(10.6, TAU_WIN, RHO_WIN, incW);
        win[shft + 0][1] = new Window(37.2, TAU_WIN, RHO_WIN, incN);
        win[shft + 0][2] = new Window(10.6, TAU_WIN, RHO_WIN, incE);
        win[shft + 1][0] = new Window(10.6, TAU_WIN, RHO_WIN, incW);
        win[shft + 1][1] = new Window(31.9, TAU_WIN, RHO_WIN, incS);
      }
      for (int i = 0; i < win.Length; i++)
      {
        for (int j = 0; j < win[i].Length; j++)
        {
          VenetianBlind blind = new VenetianBlind(25, 22.5, 0, 0, 0.66, 0.66);
          blind.SlatAngle = 0;
          win[i][j].SetShadingDevice(1, blind);
          win[i][j].ConvectiveCoefficientF = 18;
          win[i][j].ConvectiveCoefficientB = 4;
          win[i][j].LongWaveEmissivityF = win[i][j].LongWaveEmissivityB = 0.9;
        }
      }

      //多数室の作成
      MultiRooms[] mRm = new MultiRooms[FLR_NUM * 3];
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int mSft = fl * 3;

        mRm[mSft + 0] = new MultiRooms(1, zn0[fl], walls[fl], win[mSft + 0]);
        mRm[mSft + 1] = new MultiRooms(1, zn1[fl], walls[fl], win[mSft + 1]);
        mRm[mSft + 2] = new MultiRooms(4, zn2[fl], walls[fl], win[mSft + 2]);

        //ゾーンを室に登録
        mRm[mSft + 0].AddZone(0, 0);
        mRm[mSft + 0].AddZone(0, 1);
        mRm[mSft + 0].AddZone(0, 2);
        mRm[mSft + 0].AddZone(0, 3);
        mRm[mSft + 1].AddZone(0, 0);
        mRm[mSft + 1].AddZone(0, 1);
        mRm[mSft + 1].AddZone(0, 2);
        mRm[mSft + 2].AddZone(0, 0);
        mRm[mSft + 2].AddZone(1, 1);
        mRm[mSft + 2].AddZone(2, 2);
        mRm[mSft + 2].AddZone(3, 3);

        //外壁を登録
        mRm[mSft + 0].AddWall(1, 0, false); mRm[0].SetOutsideWall(0, true, incW);
        mRm[mSft + 0].AddWall(1, 1, false); mRm[0].SetOutsideWall(1, true, incW);
        mRm[mSft + 0].AddWall(2, 2, false); mRm[0].SetOutsideWall(2, true, incN);
        mRm[mSft + 0].AddWall(2, 3, false); mRm[0].SetOutsideWall(3, true, incN);
        mRm[mSft + 0].AddWall(3, 4, false); mRm[0].SetOutsideWall(4, true, incE);
        mRm[mSft + 0].AddWall(3, 5, false); mRm[0].SetOutsideWall(5, true, incE);
        mRm[mSft + 1].AddWall(1, 6, false); mRm[1].SetOutsideWall(6, true, incW);
        mRm[mSft + 1].AddWall(1, 7, false); mRm[1].SetOutsideWall(7, true, incW);
        mRm[mSft + 1].AddWall(2, 8, false); mRm[1].SetOutsideWall(8, true, incS);
        mRm[mSft + 1].AddWall(2, 9, false); mRm[1].SetOutsideWall(9, true, incS);
        mRm[mSft + 2].AddWall(2, 10, false); mRm[2].SetOutsideWall(10, true, incE);
        mRm[mSft + 2].AddWall(3, 11, false); mRm[2].SetOutsideWall(11, true, incE);
        mRm[mSft + 2].AddWall(3, 12, false); mRm[2].SetOutsideWall(12, true, incS);

        //内壁を登録
        mRm[mSft + 0].AddWall(0, 0, 13);
        mRm[mSft + 0].AddWall(0, 0, 14);
        mRm[mSft + 0].AddWall(0, 15, true); mRm[mSft + 2].AddWall(1, 15, false);
        mRm[mSft + 0].AddWall(0, 16, true); mRm[mSft + 2].AddWall(0, 16, false);
        mRm[mSft + 0].AddWall(0, 17, false); mRm[mSft + 0].UseAdjacentSpaceFactor(17, true, 0.5);
        mRm[mSft + 0].AddWall(1, 1, 18);
        mRm[mSft + 0].AddWall(1, 1, 19);
        mRm[mSft + 0].AddWall(1, 20, false); mRm[mSft + 0].UseAdjacentSpaceFactor(20, true, 0.5);
        mRm[mSft + 0].AddWall(2, 2, 21);
        mRm[mSft + 0].AddWall(2, 2, 22);
        mRm[mSft + 0].AddWall(3, 3, 23);
        mRm[mSft + 0].AddWall(3, 3, 24);
        mRm[mSft + 0].AddWall(3, 25, false); mRm[mSft + 0].UseAdjacentSpaceFactor(25, true, 0.5);
        mRm[mSft + 1].AddWall(0, 26, true); mRm[mSft + 2].AddWall(1, 26, false);
        mRm[mSft + 1].AddWall(0, 27, true); mRm[mSft + 2].AddWall(2, 27, false);
        mRm[mSft + 1].AddWall(0, 28, false); mRm[mSft + 1].UseAdjacentSpaceFactor(28, true, 0.5);
        mRm[mSft + 1].AddWall(0, 0, 29);
        mRm[mSft + 1].AddWall(0, 0, 30);
        mRm[mSft + 1].AddWall(1, 1, 31);
        mRm[mSft + 1].AddWall(1, 1, 32);
        mRm[mSft + 1].AddWall(1, 33, false); mRm[mSft + 1].UseAdjacentSpaceFactor(33, true, 0.5);
        mRm[mSft + 1].AddWall(2, 2, 34);
        mRm[mSft + 1].AddWall(2, 2, 35);
        mRm[mSft + 1].AddWall(2, 36, true); mRm[mSft + 2].AddWall(3, 36, false);
        mRm[mSft + 2].AddWall(0, 37, false); mRm[mSft + 2].UseAdjacentSpaceFactor(37, true, 0.5);
        mRm[mSft + 2].AddWall(0, 0, 38);
        mRm[mSft + 2].AddWall(0, 0, 39);
        mRm[mSft + 2].AddWall(1, 2, 40);
        mRm[mSft + 2].AddWall(1, 41, false); mRm[mSft + 2].UseAdjacentSpaceFactor(41, true, 0.5);
        mRm[mSft + 2].AddWall(1, 1, 42);
        mRm[mSft + 2].AddWall(1, 1, 43);
        mRm[mSft + 2].AddWall(2, 44, true); mRm[mSft + 2].AddWall(3, 44, false);
        mRm[mSft + 2].AddWall(2, 2, 45);
        mRm[mSft + 2].AddWall(2, 2, 46);
        mRm[mSft + 2].AddWall(3, 3, 47);
        mRm[mSft + 2].AddWall(3, 3, 48);

        //窓を登録
        mRm[mSft + 0].AddWindow(1, 0);
        mRm[mSft + 0].AddWindow(2, 1);
        mRm[mSft + 0].AddWindow(3, 2);
        mRm[mSft + 1].AddWindow(1, 0);
        mRm[mSft + 1].AddWindow(2, 1);

        //ペリメータ床に短波長優先配分
        const double SW_RATE_TO_FLOOR = 0.7;
        mRm[mSft + 0].SetSWDistributionRateToFloor(0, 18, true, SW_RATE_TO_FLOOR);
        mRm[mSft + 0].SetSWDistributionRateToFloor(1, 21, true, SW_RATE_TO_FLOOR);
        mRm[mSft + 0].SetSWDistributionRateToFloor(2, 23, true, SW_RATE_TO_FLOOR);
        mRm[mSft + 1].SetSWDistributionRateToFloor(0, 31, true, SW_RATE_TO_FLOOR);
        mRm[mSft + 1].SetSWDistributionRateToFloor(1, 34, true, SW_RATE_TO_FLOOR);
      }

      //建物モデルの作成
      BuildingThermalModel bModel = new BuildingThermalModel(mRm);
      bModel.TimeStep = 3600;

      //ゾーン間換気の設定
      const double cvRate = 150d * 1.2 / 3600d;
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int mSft = fl * 3;
        bModel.SetCrossVentilation(mSft + 0, 0, mSft + 0, 1, 9.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 0, 0, mSft + 0, 2, 32.5 * cvRate);
        bModel.SetCrossVentilation(mSft + 0, 0, mSft + 0, 3, 9.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 0, 1, mSft + 0, 2, 7.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 0, 2, mSft + 0, 3, 7.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 1, 0, mSft + 1, 1, 9.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 1, 0, mSft + 1, 2, 31.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 1, 1, mSft + 1, 2, 7.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 2, 0, mSft + 2, 1, 3.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 2, 1, mSft + 2, 2, 2.0 * cvRate);
        bModel.SetCrossVentilation(mSft + 2, 2, mSft + 2, 3, 2.0 * cvRate);
      }

      return bModel;
    }

    public static void MakeWeatherData
      (uint seed, out double[] dbt, out double[] hrt, out double[] dnr, out double[] dhr, out double[] ncr)
    {
      double[] rad;
      bool[] fcf;
      dnr = new double[8760];
      dhr = new double[8760];
      ncr = new double[8760];
      RandomWeather.MakeWeather(seed, RandomWeather.Location.Tokyo, 1, out dbt, out hrt, out rad, out fcf);
      for (int i = 0; i < hrt.Length; i++) hrt[i] *= 0.001; //g/kg→kg/kgに換算

      DateTime dt = new DateTime(2014, 1, 1, 0, 0, 0);
      Sun sun = new Sun(Sun.City.Tokyo);
      for (int i = 0; i < 8760; i++)
      {
        sun.Update(dt);
        sun.SeparateGlobalHorizontalRadiation(rad[i], Sun.SeparationMethod.Udagawa);
        dnr[i] = sun.DirectNormalRadiation;
        dhr[i] = sun.DiffuseHorizontalRadiation;
        double vp = MoistAir.GetWaterVaporPartialPressureFromHumidityRatio(hrt[i], 101.325);
        ncr[i] = Sky.GetNocturnalRadiation(dbt[i], fcf[i] ? 0 : 5, vp);
        dt = dt.AddHours(1);
      }
    }

    public static void SetHVACControl(BuildingThermalModel bModel)
    {
      //温湿度判定
      double dbt, hrt;
      DateTime dt = bModel.CurrentDateTime;
      if (6 <= dt.Month && dt.Month < 10)
      {
        dbt = 26;
        hrt = 0.00930;
      }
      else if (dt.Month == 12 || dt.Month < 4)
      {
        dbt = 22;
        hrt = 0.00656;
      }
      else
      {
        dbt = 24;
        hrt = 0.01050;
      }

      for (int i = 0; i < bModel.MultiRoom.Length; i++)
      {
        for (int j = 0; j < bModel.MultiRoom[i].ZoneNumber; j++)
        {
          bool ffloat;
          if (IsHoliday(dt)) ffloat = true;
          else if (i == 2) ffloat = (dt.Hour < 7 || 21 <= dt.Hour) || (j == 4);
          else ffloat = (dt.Hour < 8 || 21 <= dt.Hour); 
          if (ffloat)
          {
            bModel.ControlHeatSupply(i, j, 0);
            bModel.ControlWaterSupply(i, j, 0);
          }
          else
          {
            bModel.ControlDrybulbTemperature(i, j, dbt);
            bModel.ControlHumidityRatio(i, j, hrt);
          }
        }
      }
    }

    /// <summary>土日祝日か否かを判定する</summary>
    /// <param name="dt">現在の日時</param>
    /// <returns>土日祝日の場合にtrue</returns>
    public static bool IsHoliday(DateTime dt)
    {
      if (dt.DayOfWeek == DayOfWeek.Saturday) return true;
      if (dt.DayOfWeek == DayOfWeek.Sunday) return true;
      if (dt.Month == 1 && dt.Day == 2) return true;
      if (dt.Month == 1 && dt.Day == 3) return true;
      if (dt.Month == 1 && dt.Day == 9) return true;
      if (dt.Month == 2 && dt.Day == 11) return true;
      if (dt.Month == 3 && dt.Day == 21) return true;
      if (dt.Month == 4 && dt.Day == 29) return true;
      if (dt.Month == 5 && dt.Day == 3) return true;
      if (dt.Month == 5 && dt.Day == 4) return true;
      if (dt.Month == 5 && dt.Day == 5) return true;
      if (dt.Month == 7 && dt.Day == 17) return true;
      if (dt.Month == 9 && dt.Day == 18) return true;
      if (dt.Month == 9 && dt.Day == 23) return true;
      if (dt.Month == 10 && dt.Day == 9) return true;
      if (dt.Month == 11 && dt.Day == 3) return true;
      if (dt.Month == 11 && dt.Day == 23) return true;
      if (dt.Month == 12 && dt.Day == 23) return true;
      if (dt.Month == 12 && dt.Day == 29) return true;
      return false;
    }

    #endregion

    #region 熱源テスト関連

    public static void MakeHeatSourceSystem(out HeatSourceSystemModel hss,
      out AirHeatSourceModularChillersSystem ahSystem, out DirectFiredAbsorptionChillerSystem arSystem)
    {
      CentrifugalPump.ControlMethod P1_CTRL, P2_CTRL;
      if(USE_VWV_SYSTEM) P1_CTRL = P2_CTRL = CentrifugalPump.ControlMethod.ConstantPressureWithInverter;
      else P1_CTRL = P2_CTRL = CentrifugalPump.ControlMethod.ConstantPressureWithInverter;

      //空気熱源ヒートポンプシステムの作成
      double mf = 430d / 60;  //冷温水質量流量[kg/s]
      double COP_C, COP_H;
      if (IS_HIGHEFF_HSOURCE)
      {
        COP_C = 3.85;
        COP_H = 3.5;
      }
      else COP_C = COP_H = 3.0;

      AirHeatSourceModularChillers ahpChiler = new AirHeatSourceModularChillers
        (150, 7, mf, 35, 850d / 60 * 1.2, 150 / COP_C, 150, 45, mf, 7, 850d / 60 * 1.2, 150 / COP_H, 2, 1.9);
      //ahpChiler.MaximizeEfficiency = IS_HIGHEFF_HSOURCE;
      CentrifugalPump chwPump = new CentrifugalPump(150, 0.001 * mf, 140, 0.001 * mf, P1_CTRL, 1);
      CentrifugalPump hwPump = new CentrifugalPump(150, 0.001 * mf, 140, 0.001 * mf, P1_CTRL, 1);
      ahSystem = new AirHeatSourceModularChillersSystem(ahpChiler, chwPump, hwPump, 2);

      //吸収冷温水機システムの作成
      mf = 1512d / 60;  //冷温水質量流量[kg/s]
      DirectFiredAbsorptionChiller arChiller = new DirectFiredAbsorptionChiller
        (32.4d / 3600, 31.8d / 3600, 12, 7, 32, 37, 51.7, 55, mf, 2500d / 60, mf, 5.1, Boiler.Fuel.Gas13A);
      arChiller.HasSolutionInverterPump = IS_HIGHEFF_HSOURCE;
      CoolingTower cTower = new CoolingTower
        (37, 32, 27, 2693d / 60, 1783d / 60 * 1.2, CoolingTower.AirFlowDirection.CrossFlow, 7.5 / 0.92, false);
      chwPump = new CentrifugalPump(150, 0.001 * mf, 140, 0.001 * mf, P1_CTRL, 1);
      hwPump = new CentrifugalPump(150, 0.001 * mf, 140, 0.001 * mf, P1_CTRL, 1);
      CentrifugalPump cdwPump = new CentrifugalPump
        (250, 2.693 / 60, 240, 2.693 / 60, CentrifugalPump.ControlMethod.MinimumPressure, 50);
      arSystem = new DirectFiredAbsorptionChillerSystem(arChiller, chwPump, hwPump, cdwPump, cTower, 1, 1);
      arSystem.ControlCoolingWaterFlowRate = true;
      arSystem.ControlCoolingWaterTemperature = true;

      //熱源サブシステムの作成
      CentrifugalPump cp2 = new CentrifugalPump(170, 3.231 / 3d / 60d, 170, 3.231 / 3d / 60d, P2_CTRL, 1);
      PumpSystem cp2system = new PumpSystem(cp2, 170, 3.231 / 60d, 50, 3); //BugFix,2017.01.08. E.Togashi
      CentrifugalPump hp2 = new CentrifugalPump(170, 3.231 / 3d / 60d, 170, 3.231 / 3d / 60d, P2_CTRL, 1);
      PumpSystem hp2system = new PumpSystem(hp2, 170, 3.231 / 60d, 50, 3); //BugFix,2017.01.08. E.Togashi
      hss = new HeatSourceSystemModel(new IHeatSourceSubSystem[] { ahSystem, arSystem }, cp2system, hp2system);
      hss.ChilledWaterSupplyTemperatureSetpoint = 7;
      hss.HotWaterSupplyTemperatureSetpoint = 45;
      hss.SetChillingOperationSequence(0, 1);
      hss.SetChillingOperationSequence(1, 2);
      hss.SetHeatingOperationSequence(0, 1);
      hss.SetHeatingOperationSequence(1, 2);
    }

    public static void ControlHeatSourceSystem(HeatSourceSystemModel hss)
    {
      DateTime dTime = hss.CurrentDateTime;

      //冷房期間
      //if (6 <= dTime.Month && dTime.Month <= 9)
      if (5 <= dTime.Month && dTime.Month <= 10)
      {
        hss.SetOperatingMode(0, HeatSourceSystemModel.OperatingMode.Cooling);
        hss.SetOperatingMode(1, HeatSourceSystemModel.OperatingMode.Cooling);
      }
      //暖房期間
      //else if (12 <= dTime.Month || dTime.Month <= 3)
      else if (11 <= dTime.Month || dTime.Month <= 4)
      {
        hss.SetOperatingMode(0, HeatSourceSystemModel.OperatingMode.Heating);
        hss.SetOperatingMode(1, HeatSourceSystemModel.OperatingMode.Heating);
      }
      //中間期//発生しない
      else
      {
        hss.SetOperatingMode(0, HeatSourceSystemModel.OperatingMode.ShutOff);
        hss.SetOperatingMode(1, HeatSourceSystemModel.OperatingMode.ShutOff);
      }
    }

    #endregion

    #region 空調テスト関連

    public static void MakeAHUSystem(out BuildingThermalModel bModel, out AHUSystem ahuSystem, bool isCAVSystem, bool addSimpleHeatGain)
    {
      //二次側システムモデル作成
      bModel = MakeBuildingThermalModel(addSimpleHeatGain);
      AirHandlingUnit[] ahu = makeAHUs();
      ahuSystem = new AHUSystem(bModel, ahu);

      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 4;

        //CAV,VAVを作成
        const double MIN_RATE = 0.2;
        //NI系統
        AHUSystem.VolumeController[] vcNI = new AHUSystem.VolumeController[1];
        vcNI[0] = new AHUSystem.VolumeController
          (0, 0, 6212d / 3600 * 1.2, 5700d / 3600 * 1.2, 6212d / 3600 * 1.2 * MIN_RATE);
        //NP系統
        AHUSystem.VolumeController[] vcNP = new AHUSystem.VolumeController[3];
        vcNP[0] = new AHUSystem.VolumeController
          (0, 1, 1543d / 3600 * 1.2, 1433d / 3600 * 1.2, 1543d / 3600 * 1.2 * MIN_RATE);
        vcNP[1] = new AHUSystem.VolumeController
          (0, 2, 4151d / 3600 * 1.2, 3857d / 3600 * 1.2, 4151d / 3600 * 1.2 * MIN_RATE);
        vcNP[2] = new AHUSystem.VolumeController
          (0, 3, 1782d / 3600 * 1.2, 1656d / 3600 * 1.2, 1782d / 3600 * 1.2 * MIN_RATE);
        //SI系統
        AHUSystem.VolumeController[] vcSI = new AHUSystem.VolumeController[1];
        vcSI[0] = new AHUSystem.VolumeController
          (1, 0, 5730d / 3600 * 1.2, 5242d / 3600 * 1.2, 5730d / 3600 * 1.2 * MIN_RATE);
        //SP系統
        AHUSystem.VolumeController[] vcSP = new AHUSystem.VolumeController[2];
        vcSP[0] = new AHUSystem.VolumeController
          (1, 1, 1510d / 3600 * 1.2, 1393d / 3600 * 1.2, 1510d / 3600 * 1.2 * MIN_RATE);
        vcSP[1] = new AHUSystem.VolumeController
          (1, 2, 3596d / 3600 * 1.2, 3319d / 3600 * 1.2, 3596d / 3600 * 1.2 * MIN_RATE);

        if (isCAVSystem)
        {
          ahuSystem.SetCAV(shft + 0, vcNI);
          ahuSystem.SetCAV(shft + 1, vcNP);
          ahuSystem.SetCAV(shft + 2, vcSI);
          ahuSystem.SetCAV(shft + 3, vcSP);
        }
        else
        {
          ahuSystem.SetVAV(shft + 0, vcNI);
          ahuSystem.SetVAV(shft + 1, vcNP);
          ahuSystem.SetVAV(shft + 2, vcSI);
          ahuSystem.SetVAV(shft + 3, vcSP);
        }
      }
    }

    private static AirHandlingUnit[] makeAHUs()
    {
      const double MIN_FAN_RATIO = 0.2;

      AirHandlingUnit[] ahus = new AirHandlingUnit[FLR_NUM * 4];

      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 4;

        CrossFinHeatExchanger cCoil, hCoil;
        CentrifugalFan saFan, raFan;
        RotaryRegenerator regenerator;
        double msa, mra, moa;

        double dPHEX = 0;
        if (USE_REGENERATOR) dPHEX = 0.2;

        //AHU-1Iを作成
        msa = 6212d / 3600 * 1.2;
        mra = 5700d / 3600 * 1.2;
        moa = 1463d / 3600 * 1.2;
        cCoil = new CrossFinHeatExchanger(0.82, 0.910, 6, 24, msa, 27.46, 0.01206, 95,
          111d / 60, 111d / 60, 7, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 43.5, true);
        hCoil = new CrossFinHeatExchanger(0.82, 0.910, 4, 24, msa, 17.46, 0.00554, 95,
          93d / 60, 93d / 60, 50, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 41.5, true);
        saFan = new CentrifugalFan(0.85 + dPHEX, msa / 1.2, 0.85 + dPHEX, msa / 1.2, 3, true);
        raFan = new CentrifugalFan(0.35 + dPHEX, mra / 1.2, 0.35 + dPHEX, mra / 1.2, 3, true);
        saFan.MinimumRotationRatio = MIN_FAN_RATIO;
        raFan.MinimumRotationRatio = MIN_FAN_RATIO;
        regenerator = new RotaryRegenerator
          (0.767, 0.730, 0.34, 210, 1463, 1463, true, 35, 0.0195, 26, 0.0105, 0.1);
        ahus[shft + 0] = new AirHandlingUnit
          (cCoil, hCoil, AirHandlingUnit.HumidifierType.DropPervaporation, saFan, raFan, regenerator);
        ahus[shft + 0].SetOutdoorAirFlowRange(moa, msa);
        ahus[shft + 0].SetAirFlowRate(mra, msa);

        //AHU-1Pを作成
        msa = 7476d / 3600 * 1.2;
        mra = 6946d / 3600 * 1.2;
        moa = 1513d / 3600 * 1.2;
        cCoil = new CrossFinHeatExchanger(0.82, 0.910, 6, 24, msa, 27.46, 0.01206, 95,
          127d / 60, 127d / 60, 7, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 49.6, true);
        hCoil = new CrossFinHeatExchanger(0.82, 0.910, 4, 24, msa, 17.46, 0.00554, 95,
          105d / 60, 105d / 60, 50, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 46.9, true);
        saFan = new CentrifugalFan(0.8 + dPHEX, msa / 1.2, 0.8 + dPHEX, msa / 1.2, 3, true);
        raFan = new CentrifugalFan(0.3 + dPHEX, mra / 1.2, 0.3 + dPHEX, mra / 1.2, 3, true);
        saFan.MinimumRotationRatio = MIN_FAN_RATIO;
        raFan.MinimumRotationRatio = MIN_FAN_RATIO;
        regenerator = new RotaryRegenerator
          (0.767, 0.730, 0.34, 210, 1513, 1513, true, 35, 0.0195, 26, 0.0105, 0.1);
        ahus[shft + 1] = new AirHandlingUnit
          (cCoil, hCoil, AirHandlingUnit.HumidifierType.DropPervaporation, saFan, raFan, regenerator);
        ahus[shft + 1].SetOutdoorAirFlowRange(moa, msa);
        ahus[shft + 1].SetAirFlowRate(mra, msa);

        //AHU-2Pを作成
        msa = 5730d / 3600 * 1.2;
        mra = 5242d / 3600 * 1.2;
        moa = 1395d / 3600 * 1.2;
        cCoil = new CrossFinHeatExchanger(0.82, 0.910, 6, 24, msa, 27.46, 0.01206, 95,
          104d / 60, 104d / 60, 7, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 40.5, true);
        hCoil = new CrossFinHeatExchanger(0.82, 0.910, 4, 24, msa, 17.46, 0.00554, 95,
          86d / 60, 86d / 60, 50, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 38.4, true);
        saFan = new CentrifugalFan(0.9 + dPHEX, msa / 1.2, 0.9 + dPHEX, msa / 1.2, 3, true);
        raFan = new CentrifugalFan(0.4 + dPHEX, mra / 1.2, 0.4 + dPHEX, mra / 1.2, 3, true);
        saFan.MinimumRotationRatio = MIN_FAN_RATIO;
        raFan.MinimumRotationRatio = MIN_FAN_RATIO;
        regenerator = new RotaryRegenerator
          (0.767, 0.730, 0.34, 210, 1395, 1395, true, 35, 0.0195, 26, 0.0105, 0.1);
        ahus[shft + 2] = new AirHandlingUnit
          (cCoil, hCoil, AirHandlingUnit.HumidifierType.DropPervaporation, saFan, raFan, regenerator);
        ahus[shft + 2].SetOutdoorAirFlowRange(moa, msa);
        ahus[shft + 2].SetAirFlowRate(mra, msa);

        //AHU-2Pを作成
        msa = 5106d / 3600 * 1.2;
        mra = 4712d / 3600 * 1.2;
        moa = 1125d / 3600 * 1.2;
        cCoil = new CrossFinHeatExchanger(0.82, 0.910, 6, 24, msa, 27.46, 0.01206, 95,
          88d / 60, 88d / 60, 7, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 34.5, true);
        hCoil = new CrossFinHeatExchanger(0.82, 0.910, 4, 24, msa, 17.46, 0.00554, 95,
          93d / 60, 93d / 60, 50, CrossFinHeatExchanger.WaterFlowType.HalfFlow, 32.3, true);
        saFan = new CentrifugalFan(0.85 + dPHEX, msa / 1.2, 0.85 + dPHEX, msa / 1.2, 3, true);
        raFan = new CentrifugalFan(0.35 + dPHEX, mra / 1.2, 0.35 + dPHEX, mra / 1.2, 3, true);
        saFan.MinimumRotationRatio = MIN_FAN_RATIO;
        raFan.MinimumRotationRatio = MIN_FAN_RATIO;
        regenerator = new RotaryRegenerator
          (0.767, 0.730, 0.34, 210, 1125, 1125, true, 35, 0.0195, 26, 0.0105, 0.1);
        ahus[shft + 3] = new AirHandlingUnit
          (cCoil, hCoil, AirHandlingUnit.HumidifierType.DropPervaporation, saFan, raFan, regenerator);
        ahus[shft + 3].SetOutdoorAirFlowRange(moa, msa);
        ahus[shft + 3].SetAirFlowRate(mra, msa);
      }

      //共通の設定
      for (int i = 0; i < ahus.Length; i++)
      {
        if(USE_OA_COOLING) ahus[i].OutdoorAirCooling = AirHandlingUnit.OutdoorAirCoolingControl.Enthalpy;
        else ahus[i].OutdoorAirCooling = AirHandlingUnit.OutdoorAirCoolingControl.None;
        ahus[i].BypassRegenerator = !USE_REGENERATOR;
        ahus[i].MinimizeAirFlow = true;
      }      
      return ahus;
    }

    /// <summary>中間期あり</summary>
    /// <param name="bModel"></param>
    /// <param name="ahuSystem"></param>
    public static void ControlAHUSystem(BuildingThermalModel bModel, AHUSystem ahuSystem)
    {
      //温湿度判定
      double dbt = 0;
      double hrt = 0;
      DateTime dt = bModel.CurrentDateTime;
      bool midSeason = false;
      if (6 <= dt.Month && dt.Month < 10)
      {
        dbt = 26;
        hrt = 0.00930;
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.Cooling;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).UpperTemperatureLimit = 19d;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).LowerTemperatureLimit = 13d;
        }
      }
      else if (dt.Month == 12 || dt.Month < 4)
      {
        dbt = 22;
        hrt = 0.00656;
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.Heating;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).UpperTemperatureLimit = 37d;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).LowerTemperatureLimit = 27d;
          ahuSystem.Controllers[i].MinimumHumidity = hrt;
        }
      }
      else midSeason = true;

      //事務室//CAV,VAVコントローラを制御
      if ((IsHoliday(dt)) || (dt.Hour < 7 || 21 <= dt.Hour) || midSeason)
        for (int i = 0; i < ahuSystem.AHUs.Length; i++) ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.ShutOff;
      else
      {
        //外気カットウォーミングアップ設定
        if (dt.Hour == 7) for (int i = 0; i < ahuSystem.AHUs.Length; i++) ahuSystem.SetOutdoorAirFlow(i, 0, 0);
        else
        {
          for (int fl = 0; fl < FLR_NUM; fl++)
          {
            int shft = fl * 4;
            ahuSystem.SetOutdoorAirFlow(shft + 0, 1463d / 3600 * 1.2, 1463d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 1, 1513d / 3600 * 1.2, 1513d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 2, 1395d / 3600 * 1.2, 1395d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 3, 1125d / 3600 * 1.2, 1125d / 3600 * 1.2);
          }
        }

        //ゾーン温度を設定(VAV用)
        for (int fl = 0; fl < FLR_NUM; fl++)
        {
          int shft = fl * 4;

          ahuSystem.ControlZoneTemperature(shft + 0, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 1, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 2, dbt);
          ahuSystem.ControlZoneTemperature(shft + 2, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 3, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 3, 1, dbt);
        }
        //(CAV用)
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].IsRATemperatureControl = true;
          ahuSystem.Controllers[i].SetpointTemperature = dbt;
        }
      }

      //廊下・便所は直接に熱負荷計算
      for (int i = 0; i < bModel.MultiRoom[2].ZoneNumber; i++)
      {
        bool ffloat = (IsHoliday(dt)) || (dt.Hour < 7 || 21 <= dt.Hour) || (i == 3) || midSeason;
        if (ffloat)
        {
          bModel.ControlHeatSupply(2, i, 0);
          bModel.ControlWaterSupply(2, i, 0);
        }
        else
        {
          bModel.ControlDrybulbTemperature(2, i, dbt);
          bModel.ControlHumidityRatio(2, i, hrt);
        }
      }

      //風の流れを設定
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 3;
        if (!IsHoliday(dt) && (dt.Hour < 8 || 21 <= dt.Hour))
        {

          bModel.SetAirFlow(shft + 0, 0, shft + 2, 1, 985d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 1, 0, shft + 2, 1, 940d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 2, 1, shft + 2, 2, 1925d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 2, 2, shft + 2, 3, 1925d / 3600 * 1.2);

        }
        else
        {
          const double cvRate = 150d * 1.2 / 3600d;
          bModel.SetAirFlow(shft + 2, 1, shft + 2, 2, 0);
          bModel.SetAirFlow(shft + 2, 2, shft + 2, 3, 0);
          bModel.SetCrossVentilation(shft + 2, 1, shft + 2, 2, 2.0 * cvRate);
          bModel.SetCrossVentilation(shft + 2, 2, shft + 2, 3, 2.0 * cvRate);
        }
      }      
    }

    /// <summary>中間期無し</summary>
    /// <param name="bModel"></param>
    /// <param name="ahuSystem"></param>
    public static void ControlAHUSystem_NoMidSeason(BuildingThermalModel bModel, AHUSystem ahuSystem)
    {
      //温湿度判定
      double dbt = 0;
      double hrt = 0;
      DateTime dt = bModel.CurrentDateTime;
      bool midSeason = false;
      if (5 <= dt.Month && dt.Month <= 10)
      {
        dbt = 26;
        hrt = 0.00930;
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.Cooling;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).UpperTemperatureLimit = 19d;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).LowerTemperatureLimit = 13d;
        }
      }
      else if (11 <= dt.Month || dt.Month <= 4)
      {
        dbt = 22;
        hrt = 0.00656;
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.Heating;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).UpperTemperatureLimit = 37d;
          ((AirHandlingUnit)ahuSystem.AHUs[i]).LowerTemperatureLimit = 27d;
          ahuSystem.Controllers[i].MinimumHumidity = hrt;
        }
      }
      else midSeason = true;

      //事務室//CAV,VAVコントローラを制御
      if ((IsHoliday(dt)) || (dt.Hour < 7 || 21 <= dt.Hour) || midSeason)
        for (int i = 0; i < ahuSystem.AHUs.Length; i++) ahuSystem.Controllers[i].Mode = AHUSystem.OperatingMode.ShutOff;
      else
      {
        //外気カットウォーミングアップ設定
        if (dt.Hour == 7) for (int i = 0; i < ahuSystem.AHUs.Length; i++) ahuSystem.SetOutdoorAirFlow(i, 0, 0);
        else
        {
          for (int fl = 0; fl < FLR_NUM; fl++)
          {
            int shft = fl * 4;
            ahuSystem.SetOutdoorAirFlow(shft + 0, 1463d / 3600 * 1.2, 1463d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 1, 1513d / 3600 * 1.2, 1513d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 2, 1395d / 3600 * 1.2, 1395d / 3600 * 1.2);
            ahuSystem.SetOutdoorAirFlow(shft + 3, 1125d / 3600 * 1.2, 1125d / 3600 * 1.2);
          }
        }

        //ゾーン温度を設定(VAV用)
        for (int fl = 0; fl < FLR_NUM; fl++)
        {
          int shft = fl * 4;

          ahuSystem.ControlZoneTemperature(shft + 0, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 1, dbt);
          ahuSystem.ControlZoneTemperature(shft + 1, 2, dbt);
          ahuSystem.ControlZoneTemperature(shft + 2, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 3, 0, dbt);
          ahuSystem.ControlZoneTemperature(shft + 3, 1, dbt);
        }
        //(CAV用)
        for (int i = 0; i < ahuSystem.AHUs.Length; i++)
        {
          ahuSystem.Controllers[i].IsRATemperatureControl = true;
          ahuSystem.Controllers[i].SetpointTemperature = dbt;
        }
      }

      //廊下・便所は直接に熱負荷計算
      for (int i = 0; i < bModel.MultiRoom[2].ZoneNumber; i++)
      {
        bool ffloat = (IsHoliday(dt)) || (dt.Hour < 7 || 21 <= dt.Hour) || (i == 3) || midSeason;
        if (ffloat)
        {
          bModel.ControlHeatSupply(2, i, 0);
          bModel.ControlWaterSupply(2, i, 0);
        }
        else
        {
          bModel.ControlDrybulbTemperature(2, i, dbt);
          bModel.ControlHumidityRatio(2, i, hrt);
        }
      }

      //風の流れを設定
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 3;
        if (!IsHoliday(dt) && (dt.Hour < 8 || 21 <= dt.Hour))
        {

          bModel.SetAirFlow(shft + 0, 0, shft + 2, 1, 985d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 1, 0, shft + 2, 1, 940d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 2, 1, shft + 2, 2, 1925d / 3600 * 1.2);
          bModel.SetAirFlow(shft + 2, 2, shft + 2, 3, 1925d / 3600 * 1.2);

        }
        else
        {
          const double cvRate = 150d * 1.2 / 3600d;
          bModel.SetAirFlow(shft + 2, 1, shft + 2, 2, 0);
          bModel.SetAirFlow(shft + 2, 2, shft + 2, 3, 0);
          bModel.SetCrossVentilation(shft + 2, 1, shft + 2, 2, 2.0 * cvRate);
          bModel.SetCrossVentilation(shft + 2, 2, shft + 2, 3, 2.0 * cvRate);
        }
      }
    }

    public static void ApplyCO2Control(AHUSystem ahuSystem, TenantLoad[] tLoads)
    {
      if (!USE_CO2CNTRL) return;
      
      AirHandlingUnit ahu;
      for (int fl = 0; fl < FLR_NUM; fl++)
      {
        int shft = fl * 4;
        
        //Nゾーン
        double vol = (ahuSystem.AHUs[shft + 0].OAFlowRate + ahuSystem.AHUs[shft + 1].OAFlowRate) / 1.2;
        tLoads[fl * 2 + 0].UpdateCO2State(vol);
        double mVol = (ahuSystem.AHUs[shft + 0].MaxOAFlowRate + ahuSystem.AHUs[shft + 1].MaxOAFlowRate) / 1.2;
        double oaRatio;
        if (mVol == 0) oaRatio = 0;
        else oaRatio = Math.Max(0, tLoads[fl * 2 + 0].OAVolumeSetpoint / mVol);
        ahu = (AirHandlingUnit)ahuSystem.AHUs[shft + 0];
        ahu.SetOutdoorAirFlowRange(1463d / 3600 * 1.2 * oaRatio, ahu.MaxOAFlowRate);
        ahu = (AirHandlingUnit)ahuSystem.AHUs[shft + 1];
        ahu.SetOutdoorAirFlowRange(1513d / 3600 * 1.2 * oaRatio, ahu.MaxOAFlowRate);

        //Sゾーン
        vol = (ahuSystem.AHUs[shft + 2].OAFlowRate + ahuSystem.AHUs[shft + 3].OAFlowRate) / 1.2;
        tLoads[fl * 2 + 1].UpdateCO2State(vol);
        mVol = (ahuSystem.AHUs[shft + 2].MaxOAFlowRate + ahuSystem.AHUs[shft + 3].MaxOAFlowRate) / 1.2;
        if (mVol == 0) oaRatio = 0;
        else oaRatio = Math.Max(0, tLoads[fl * 2 + 1].OAVolumeSetpoint / mVol);
        ahu = (AirHandlingUnit)ahuSystem.AHUs[shft + 2];
        ahu.SetOutdoorAirFlowRange(1395d / 3600 * 1.2 * oaRatio, ahu.MaxOAFlowRate);
        ahu = (AirHandlingUnit)ahuSystem.AHUs[shft + 3];
        ahu.SetOutdoorAirFlowRange(1125d / 3600 * 1.2 * oaRatio, ahu.MaxOAFlowRate);
      }
    }

    #endregion

    #region 内部発熱要素クラス

    /// <summary>内部発熱クラス</summary>
    public class MyHeatGain : IHeatGain
    {

      /// <summary>事務室か否か</summary>
      private bool isOffice;

      /// <summary>最大人数[人]を取得する</summary>
      public double MaxOccupancy { get; private set; }

      /// <summary>最大照明発熱[W]を取得する</summary>
      public double MaxLightingLoad { get; private set; }

      /// <summary>最大機器発熱[W]を取得する</summary>
      public double MaxPlugLoad { get; private set; }

      /// <summary>人数[人]を取得する</summary>
      public double Occupancy { get; private set; }

      /// <summary>照明発熱[W]を取得する</summary>
      public double LightingLoad { get; private set; }

      /// <summary>機器発熱[W]を取得する</summary>
      public double PlugLoad { get; private set; }

      /// <summary>現在時刻</summary>
      private DateTime currentDT;

      /// <summary>初期化する</summary>
      /// <param name="maxOccupancy">最大人数[人]</param>
      /// <param name="maxLightingLoad">最大照明発熱[W]</param>
      /// <param name="maxPlugLoad">最大機器発熱[W]</param>
      /// <param name="isOffice">事務室か否か</param>
      public MyHeatGain(double maxOccupancy, double maxLightingLoad, double maxPlugLoad, bool isOffice)
      {
        MaxOccupancy = maxOccupancy;
        MaxLightingLoad = maxLightingLoad;
        MaxPlugLoad = maxPlugLoad;
        this.isOffice = isOffice;
      }

      /// <summary>顕熱取得[W]の内、対流成分を取得する</summary>
      /// <param name="zone">発熱要素が属するゾーン</param>
      /// <returns>顕熱取得の対流成分[kW]</returns>
      public double GetConvectiveHeatGain(ImmutableZone zone)
      {
        updateLoad(zone.MultiRoom.CurrentDateTime);
        return (LightingLoad + PlugLoad + Occupancy * 80) * 0.4;
      }

      /// <summary>顕熱取得[W]の内、放射成分を取得する</summary>
      /// <param name="zone">発熱要素が属するゾーン</param>
      /// <returns>顕熱取得の放射成分[kW]</returns>
      public double GetRadiativeHeatGain(ImmutableZone zone)
      {
        updateLoad(zone.MultiRoom.CurrentDateTime);
        return (LightingLoad + PlugLoad + Occupancy * 80) * 0.6;
      }

      /// <summary>発生水分[kg/s]を取得する</summary>
      /// <param name="zone">発熱要素が属するゾーン</param>
      /// <returns>発生水分[kg/s]</returns>
      public double GetWaterGain(ImmutableZone zone)
      {
        updateLoad(zone.MultiRoom.CurrentDateTime);
        return Occupancy * 40 / 2500000d;
      }

      /// <summary>負荷を更新する</summary>
      /// <param name="dt">現在の日時</param>
      private void updateLoad(DateTime dt)
      {
        if (currentDT == dt) return;
        else currentDT = dt;

        double rLt, rOcc, rPlg;
        if (isOffice) //事務室の場合
        {
          if (IsHoliday(dt) || (dt.Hour < 8 || 21 <= dt.Hour))
          {
            rOcc = 0;
            rLt = 0;
            rPlg = 0.25;
          }
          else if (20 <= dt.Hour && dt.Hour < 21)
          {
            rOcc = 0.2;
            rLt = 0.8;
            rPlg = 0.5;
          }
          else if (19 <= dt.Hour && dt.Hour < 20)
          {
            rOcc = 0.3;
            rLt = 1.0;
            rPlg = 0.5;
          }
          else if (18 <= dt.Hour && dt.Hour < 19)
          {
            rOcc = 0.5;
            rLt = 1.0;
            rPlg = 1.0;
          }
          else if (12 <= dt.Hour && dt.Hour < 13)
          {
            rOcc = 0.6;
            rLt = 0.5;
            rPlg = 0.8;
          }
          else
          {
            rOcc = 1.0;
            rLt = 1.0;
            rPlg = 1.0;
          }
        }
        else //廊下などの場合
        {
          rOcc = 0;
          rLt = 0;
          rPlg = 0.0;
          if (!IsHoliday(dt) && 8 <= dt.Hour && dt.Hour < 21)
          {
            rOcc = 1.0;
            rLt = 1.0;
          }
        }
        LightingLoad = MaxLightingLoad * rLt;
        PlugLoad = MaxPlugLoad * rPlg;
        Occupancy = MaxOccupancy * rOcc;
      }
    }

    #endregion

    /// <summary>気象データを作成する</summary>
    /// <param name="hasDataPath">HASP形式データファイルへのパス</param>
    /// <param name="dbt">乾球温度リスト</param>
    /// <param name="hrt">絶対湿度リスト</param>
    /// <param name="dnr">法線面直達日射リスト</param>
    /// <param name="dhr">水平面全天日射リスト</param>
    /// <param name="ncr">夜間放射リスト</param>
    public static void LoadWeatherData
      (string hasDataPath, out double[] dbt, out double[] hrt, out double[] dnr, out double[] dhr, out double[] ncr)
    {
      dbt = new double[8760];
      hrt = new double[8760];
      dnr = new double[8760];
      dhr = new double[8760];
      ncr = new double[8760];
      using (StreamReader sReader = new StreamReader(hasDataPath))
      {
        string buff = sReader.ReadToEnd();
        string[] lines = WeatherConverter.HASPtoCSV(buff).Split
          (new[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

        for (int i = 0; i < 8760; i++)
        {
          string[] sbf = lines[i + 1].Split(',');
          dbt[i] = double.Parse(sbf[3]);
          hrt[i] = double.Parse(sbf[4]) / 1000d;
          dnr[i] = double.Parse(sbf[5]) * 1e6 / 3600d;
          dhr[i] = double.Parse(sbf[6]) * 1e6 / 3600d;
          ncr[i] = double.Parse(sbf[7]) * 1e6 / 3600d;
        }
      }
    }

  }

}
