﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Popolo.ThermalLoad;
using Popolo.BuildingOccupant;

namespace DDCF
{
  /// <summary>テナント発熱クラス</summary>
  public class TenantLoad: IHeatGain
  {

    #region 定数宣言

    /// <summary>照明発熱[W/m2]</summary>
    private const double LIGHTING_LOAD = 12;

    /// <summary>機器発熱[W/人]</summary>
    private const double PLUG_LOAD = 120;

    /// <summary>小便器洗浄量[L/回]</summary>
    private const double U_FLUSH = 0.8; //2.8 -> 0.8  //TOTO UFH508（\78,000）とUFS910（\124,000）複合単価東京

    /// <summary>大便器洗浄量[L/回]</summary>
    private const double C_FLUSH = 3.8; //8.0 -> 3.8  //

    /// <summary>外気CO2濃度[ppm]</summary>
    private const double OA_CO2LEVEL = 400;

    #endregion

    #region インスタンス変数

    /// <summary>現在の日時</summary>
    private DateTime cTime;

    /// <summary>熱負荷[W]</summary>
    private double scLoad, srLoad, lLoad;

    #endregion

    #region プロパティ

    /// <summary>テナントを設定・取得する</summary>
    public OfficeTenant Tenant { get; set; }

    /// <summary>CO2濃度[ppm]を取得する</summary>
    public double CO2Level { get; private set; } = OA_CO2LEVEL;

    /// <summary>外気導入量設定値[m3/s]を取得する</summary>
    public double OAVolumeSetpoint { get; private set; }

    #endregion

    #region コンストラクタ

    /// <summary>インスタンスを初期化する</summary>
    /// <param name="tenant">テナント</param>
    public TenantLoad(OfficeTenant tenant)
    {
      Tenant = tenant;
    }

    #endregion

    #region インスタンスメソッド

    /// <summary>便所洗浄水量[m3/day]を計算する</summary>
    /// <returns>便所洗浄水量[m3/day]</returns>
    public double CalculateFlushWater()
    {
      if (Tenant == null) return 0;

      double mSum = 0;
      double fSum = 0;
      foreach (ImmutableOfficeWorker ow in Tenant.OfficeWorkers)
      {
        if (ow.IsMale) mSum += ow.CalculateStayInOfficeMinutes();
        else fSum += ow.CalculateStayInOfficeMinutes();
      }
      return 0.001 *
        (fSum / 60d * 0.55 * C_FLUSH + (mSum / 60d) * (0.3 * U_FLUSH + 0.075 * C_FLUSH));
    }

    /// <summary>日スケジュールを更新</summary>
    /// <param name="dTime"></param>
    public void UpdateDailySchedule(DateTime dTime)
    {
      //日付が変わった場合には日スケジュールを更新
      if (dTime.Day != cTime.Day && Tenant != null)
        Tenant.UpdateDailySchedule(dTime);
      cTime = dTime;
    }

    /// <summary>CO2関連の情報を更新する</summary>
    /// <param name="oaFlowRate">外気導入量[m3/s]</param>
    public void UpdateCO2State(double oaFlowRate)
    {
      if (Tenant == null)
      {
        CO2Level = 0;
        return;
      }

      double vol = Tenant.FloorArea * 2.7;
      if (oaFlowRate == 0) oaFlowRate = vol * 0.05 * 3600;  //漏気0.05回/h
      double bf = -oaFlowRate / vol * 3600d;
      double cg = 0.2 * Tenant.StayWorkerNumber / 3600d;
      CO2Level = OA_CO2LEVEL + (CO2Level - OA_CO2LEVEL) * Math.Exp(bf) + cg / oaFlowRate * (1 - Math.Exp(bf));
      
      OAVolumeSetpoint = Math.Min(1.0, Math.Max(0.0, (vol * (1000 - CO2Level) - cg * 3600) / ((OA_CO2LEVEL - 1000) * 3600)));
    }

    #endregion

    #region IHeatGain実装

    /// <summary>必要に応じて状態を更新する</summary>
    /// <param name="zone">ゾーン</param>
    private void updateStatus(ImmutableZone zone)
    {
      //テナント不在の場合には負荷は0
      if (Tenant == null)
      {
        scLoad = srLoad = lLoad = 0;
        return;
      }

      //日時が変わった場合
      if (zone.MultiRoom.CurrentDateTime != cTime)
      {
        //日付が変わった場合には日スケジュールを更新
        if (zone.MultiRoom.CurrentDateTime.Day != cTime.Day)
          Tenant.UpdateDailySchedule(zone.MultiRoom.CurrentDateTime);

        cTime = zone.MultiRoom.CurrentDateTime;

        //人数と負荷を更新
        Tenant.UpdateStatus(cTime);
        //冬：22C / 50% / 0.6clo / 0.3m/s 相当 ← Clo値低い。。。
        if (cTime.Month == 12 || cTime.Month <= 3)
        {
          scLoad = (PLUG_LOAD + 98.0) * 0.4 * Tenant.StayWorkerNumber;
          srLoad = (PLUG_LOAD + 98.0) * 0.6 * Tenant.StayWorkerNumber;
          lLoad = 25.2 / 2500000 * Tenant.StayWorkerNumber;
        }
        //夏：26C / 50% / 0.6clo / 0.3m/s 相当
        else if (6 <= cTime.Month && cTime.Month <= 9)
        {
          scLoad = (PLUG_LOAD + 76.3) * 0.4 * Tenant.StayWorkerNumber;
          srLoad = (PLUG_LOAD + 76.3) * 0.6 * Tenant.StayWorkerNumber;
          lLoad = 40.7 / 2500000 * Tenant.StayWorkerNumber;
        }
        //その他：24C / 50% / 0.6clo / 0.3m/s 相当
        else
        {
          scLoad = (PLUG_LOAD + 91.2) * 0.4 * Tenant.StayWorkerNumber;
          srLoad = (PLUG_LOAD + 91.2) * 0.6 * Tenant.StayWorkerNumber;
          lLoad = 25.8 / 2500000 * Tenant.StayWorkerNumber;
        }

        //床面積あたりとする
        scLoad /= Tenant.FloorArea;
        srLoad /= Tenant.FloorArea;
        lLoad /= Tenant.FloorArea;

        //照明発熱を加算:一人でも残っていれば照明ON
        //if(cTime.DayOfWeek != DayOfWeek.Saturday && cTime.DayOfWeek != DayOfWeek.Sunday && 8 <= cTime.Hour && cTime.Hour <= 20) //debug
        if (0 < Tenant.StayWorkerNumber)  
        {
          scLoad += LIGHTING_LOAD * 0.4;
          srLoad += LIGHTING_LOAD * 0.6;
        }
      }
    }

    /// <summary>顕熱取得[W]の内、対流成分を取得する</summary>
    /// <param name="zone">ゾーン</param>
    /// <returns>顕熱取得[W]の内、対流成分</returns>
    public double GetConvectiveHeatGain(ImmutableZone zone)
    {
      updateStatus(zone);
      return scLoad * zone.FloorArea;
    }

    /// <summary>顕熱取得[W]の内、放射成分を取得する</summary>
    /// <param name="zone">ゾーン</param>
    /// <returns>顕熱取得[W]の内、放射成分</returns>
    public double GetRadiativeHeatGain(ImmutableZone zone)
    {
      updateStatus(zone);
      return srLoad * zone.FloorArea;
    }

    /// <summary>発生水分[kg/s]を取得する</summary>
    /// <param name="zone">ゾーン</param>
    /// <returns>発生水分[kg/s]</returns>
    public double GetWaterGain(ImmutableZone zone)
    {
      updateStatus(zone);
      return lLoad * zone.FloorArea;
    }

    #endregion

  }
}
