﻿using System;

using Popolo.Numerics;

namespace DDCF
{
  /// <summary>テナント入退室過程</summary>
  public class TenantProcess
  {

    #region 定数宣言

    /// <summary>ペナルティ無し事前告知月数[月]</summary>
    private const uint ANNOUNCEMENT_MONTH = 6;

    /// <summary>契約月数[月]</summary>
    private const uint CONTRACT_TERM = 24;

    #endregion

    #region インスタンス変数

    /// <summary>乱数生成器</summary>
    private MersenneTwister mRnd;
    
    #endregion

    #region  プロパティ
    
    /// <summary>探索月数の平均を設定・取得する</summary>
    public int SearchingMonth_Average { get; set; } = 3;

    /// <summary>探索月数の分散を設定・取得する</summary>
    public int SearchingMonth_Deviation { get; set; } = 2 * 2;

    /// <summary>契約月数丁度に退去する可能性[-]を設定・取得する</summary>
    public double TermLeavingProbability { get; set; } = 0.25;
    
    #endregion

    #region コンストラクタ

    /// <summary>コンストラクタ</summary>
    /// <param name="seed">乱数シード</param>
    public TenantProcess(uint seed)
    {
      mRnd = new MersenneTwister(seed);
    }

    #endregion

    #region インスタンスメソッド

    /// <summary>テナント入退室過程を確率的に生成する</summary>
    /// <param name="month">計算期間</param>
    /// <param name="currentContract">現在の契約経過月数（負の値で空室）</param>
    /// <returns>テナント入退室過程</returns>
    /// <remarks>
    /// 同一テナント契約更新の場合には1世代上げる。
    /// 別テナントが新規に入居の場合には2世代上げる。
    /// 空室の場合は0。
    /// </remarks>
    public int[] MakeProcess(uint month, int currentContract)
    {
      int generation = 1;
      int cMonth = 0;
      int[] process = new int[month];

      //0時点でテナントが入居している場合
      if (0 <= currentContract)
      {
        int remain = (int)(CONTRACT_TERM - currentContract);

        //残存月数が告知期限以下の場合は契約更新
        if (remain <= ANNOUNCEMENT_MONTH)
        {
          for (int i = 0; i < remain; i++)
          {
            if (cMonth < month) process[cMonth] = generation;
            cMonth++;
          }
        }
        //1世代テナントの退去告知可能性を検討
        else
        {
          int lm = leavingMonth(mRnd, remain, ANNOUNCEMENT_MONTH, TermLeavingProbability);
          //契約更新
          if (lm == remain)
          {
            for (int i = 0; i < remain; i++)
            {
              if (cMonth < month) process[cMonth] = generation;
              cMonth++;
            }
          }
          //事前告知・契約終了
          else
          {
            //1世代テナントの入居期間
            for (int i = 0; i < lm + ANNOUNCEMENT_MONTH; i++)
            {
              if (cMonth < month) process[cMonth] = generation;
              cMonth++;
            }
            //テナント探索期間
            int sm = Math.Max((int)ANNOUNCEMENT_MONTH, searchingMonth(mRnd, SearchingMonth_Average, SearchingMonth_Deviation));
            for (int i = 0; i < sm - ANNOUNCEMENT_MONTH; i++)
            {
              if (cMonth < month) process[cMonth] = 0;
              cMonth++;
            }
            generation++;
          }
        }
      }
      else
      {
        //テナントがいない場合には1世代テナントは0時点で退出とみなす
        int sm = Math.Max((int)ANNOUNCEMENT_MONTH, searchingMonth(mRnd, SearchingMonth_Average, SearchingMonth_Deviation));
        for (int i = 0; i < sm; i++)
        {
          if (cMonth < month) process[cMonth] = 0;
          cMonth++;
        }
        generation++;
      }

      while (cMonth <= month)
      {
        //テナントの退去告知可能性を検討
        int lm = leavingMonth(mRnd, (int)CONTRACT_TERM, ANNOUNCEMENT_MONTH, TermLeavingProbability);
        //契約更新
        if (lm == CONTRACT_TERM)
        {
          for (int i = 0; i < CONTRACT_TERM; i++)
          {
            if (cMonth < month) process[cMonth] = generation;
            cMonth++;
          }
        }
        //事前告知・契約終了
        else
        {
          //1世代テナントの入居期間
          for (int i = 0; i < lm + ANNOUNCEMENT_MONTH; i++)
          {
            if (cMonth < month) process[cMonth] = generation;
            cMonth++;
          }
          //検討月数を超えた場合は計算打ち切り
          if (month < cMonth) break;
          //テナント探索期間
          int sm = Math.Max((int)ANNOUNCEMENT_MONTH, searchingMonth(mRnd, SearchingMonth_Average, SearchingMonth_Deviation));
          for (int i = 0; i < sm - ANNOUNCEMENT_MONTH; i++)
          {
            if(cMonth < month) process[cMonth] = 0;
            cMonth++;
          }
          generation++;
        }
      }
      return process;
    }
    
    #endregion

    #region staticメソッド

    /// <summary>テナント探索月数を確率的に発生させる</summary>
    /// <param name="random">乱数生成インスタンス</param>
    /// <param name="average">平均探索月数</param>
    /// <param name="deviation">探索月数分散</param>
    /// <returns>テナント探索月数</returns>
    private static int searchingMonth(MersenneTwister random, double average, double deviation)
    {
      double p = 1 - average / deviation;
      double a = average * average / (deviation - average);

      double prob = Math.Pow(1 - p, a);
      double rnd = random.NextDouble();

      double lastProb = prob;
      int num = 0;
      while (prob < rnd)
      {
        lastProb *= ((a + num) / (num + 1)) * p;
        prob += lastProb;
        num++;
      }
      return num;
    }

    /// <summary>テナント退出告知までの月数を確率的に発生させる</summary>
    /// <param name="random">乱数生成インスタンス</param>
    /// <param name="contractTerm">契約月数</param>
    /// <param name="announcementMonth">事前告知の月数</param>
    /// <param name="termLeavingProbability">契約月数丁度に退去する確率</param>
    /// <returns>テナント退出告知までの月数</returns>
    private static int leavingMonth
      (MersenneTwister random, int contractTerm, uint announcementMonth, double termLeavingProbability)
    {
      double[] prob = new double[contractTerm - announcementMonth];
      double ongo = prob[0] = termLeavingProbability;
      for (int i = 1; i < prob.Length; i++)
      {
        prob[i] = prob[i - 1] * termLeavingProbability;
        ongo += prob[i];
      }
      ongo = Math.Max(0, 1 - ongo);

      double rnd = random.NextDouble();

      //契約更新の場合
      if (rnd < ongo) return (int)contractTerm;

      for (int i = 0; i < prob.Length; i++)
      {
        ongo += prob[i];
        if (rnd < ongo) return (int)(contractTerm - announcementMonth - i);
      }
      throw new Exception();
    }

    #endregion

  }
}
